<?php

namespace app\components;

use Yii;
use yii\base\Component;
use yii\base\InvalidConfig\Exception;
use yii\helpers\ArrayHelper;

/**
 * ClientSiesa.php
 *
 * PHP Version 5.3.0
 * 
 * @category category
 * @package package
 * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version SVN: $Id$
 * @link http://www.ingeneo.com.co
 *  
 */

/**
 * ClientSiesa.php
 *
 * // SUS COMENTARIOS AQUI
 *
 * @category category
 * @package package
 * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version Release: $Id$
 * @link http://www.ingeneo.com.co
 * 
 */
class ClientSiesa extends Component {

    /**
     * @const Valor por defecto para las sucursales
     */
    const CODE_BRANCH_DEFAULT = '001';

    /**
     * @const Nombre Categoria por defecto
     */
    const CATEGORY_DEFAULT_NAME = 'Otros';

    /**
     * @const Codigo Categoria por defecto
     */
    const CATEGORY_DEFAULT_CODE = 4;

    /**
     * Constructor de la clase
     * 
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version SVN: $Id$
     * @copyright 2017 INGENEO S.A.S.
     * @return void
     * @access public
     * 
     */
    public function __construct() {
        
    }

    /**
     * Funcion desctructora 
     * 
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version SVN: $Id$
     * @copyright 2017 INGENEO S.A.S. 
     * @return void
     * @access public
     * 
     */
    public function __destruct() {
        
    }

    /**
     * Metodo que permite obtener el listado de clientes.
     * Retorna un objeto de tipo array de la siguiente forma
     * array(
     *       '890900842' => 'C.C. F COMFENALCO ANTIOQUIA',
     * );
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getListClient() {
        $arrayClient = array();
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "
        SELECT f200_nit, f200_razon_social
        FROM t200_mm_terceros
        WHERE f200_nit IS NOT NULL AND f200_ind_estado = 1 AND f200_ind_cliente = 1
        ORDER BY f200_razon_social ASC";
        $raw = Yii::$app->dbsql->createCommand($sql)->queryAll();
        if (count($raw) > 0) {
            foreach ($raw as $value) {
                $arrayClient[$value['f200_nit']] = $value['f200_nit'] . ' - ' . $value['f200_razon_social'];
            }
        }
        return $arrayClient;
    }

    /**
     * Retorna un array con la informacion (Nit,Razon social) del cliente
     * 
     * @param string $cliente String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getCliente($cliente) {
        # Consulra a la base de datos SQL server -------------------------------
        if(!empty(Yii::$app->user->identity->identificacion)){
            $sql = "
            SELECT cli.f200_nit as nit, (cli.f200_nit + ' - ' + cli.f200_razon_social) as razon_social
            FROM t201_mm_clientes AS suc
            JOIN t200_mm_terceros AS cli ON cli.f200_rowid = suc.f201_rowid_tercero
            JOIN t210_mm_vendedores AS ven ON ven.f210_id = suc.f201_id_vendedor
            JOIN t200_mm_terceros AS dven ON dven.f200_rowid = ven.f210_rowid_tercero
            WHERE dven.f200_nit = '".Yii::$app->user->identity->identificacion."'
               AND (cli.f200_nit LIKE '%$cliente%' OR cli.f200_razon_social LIKE '%$cliente%')   
            GROUP BY cli.f200_nit, cli.f200_razon_social
            ORDER BY cli.f200_razon_social ASC
            ";
        }else{
            $sql = "
            SELECT f200_nit as nit, (f200_nit + ' - ' + f200_razon_social) as razon_social
            FROM t200_mm_terceros
            WHERE f200_nit IS NOT NULL AND f200_ind_estado = 1 AND f200_ind_interno = 1
            AND (f200_nit LIKE '%$cliente%' OR f200_razon_social LIKE '%$cliente%') 
            GROUP BY f200_nit, f200_razon_social
            ORDER BY f200_razon_social ASC";
        }
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }

    /**
     * Retorna el listado de sucursales asociadas a un nit
     * 
     * @param integer $nit Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getSucursales($nit) {
        # Consulra a la base de datos SQL server -------------------------------
        if(!empty(Yii::$app->user->identity->identificacion)){
            $sql = "
            SELECT suc.f201_id_sucursal as cod_sucursal, (suc.f201_id_sucursal + ' - ' + suc.f201_descripcion_sucursal) as nombre_sucursal
            FROM t201_mm_clientes AS suc
            JOIN t200_mm_terceros AS cli ON cli.f200_rowid = suc.f201_rowid_tercero
            JOIN t210_mm_vendedores AS ven ON ven.f210_id = suc.f201_id_vendedor
            JOIN t200_mm_terceros AS dven ON dven.f200_rowid = ven.f210_rowid_tercero
            WHERE dven.f200_nit = '".Yii::$app->user->identity->identificacion."' 
                AND cli.f200_nit = '" . $nit . "' AND cli.f200_ind_estado = '1'
            GROUP BY suc.f201_id_sucursal, suc.f201_descripcion_sucursal
            ORDER BY suc.f201_descripcion_sucursal ASC
            ";
        }else{
            $sql = "SELECT f201_id_sucursal as cod_sucursal, 
            (f201_id_sucursal + ' - ' + f201_descripcion_sucursal) as nombre_sucursal
            FROM t201_mm_clientes
            LEFT JOIN t200_mm_terceros AS suc ON f200_rowid = f201_rowid_tercero
            WHERE f200_nit ='" . $nit . "' AND f200_ind_estado = '1'
            GROUP BY f201_id_sucursal, f201_descripcion_sucursal
            ORDER BY f201_descripcion_sucursal ASC";
        }
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }

    /**
     * Metodo para retornar el nombre del cliente
     * 
     * @param int $nit NIT del cliente
     * 
     * @return string
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     * 
     */
    public function getNameClient($nit) {
        $listClient = $this->getListClient();
        return $listClient[$nit];
    }

    /**
     * Metodo para obtener las sucursales segun el nit
     * de la empresa
     * Retorna un array de la siguiente forma
     * array(
     *     '002' => 'Las americas',
     *     '003' => 'Frontino',
     *     '005' => 'Las vegas',
     *     '006' => 'Bello',
     *     );
     * 
     * @param int $nit    Nit de la empresa
     * @param int $idUser Identificador del usuario registrado
     *  
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     * 
     */
    public static function getListBranch($nit, $idUser) {
        $listBranchDB = array();
        $listBranch = array();
        if (!empty($nit) && is_numeric($nit) && !empty($idUser) && is_numeric($idUser)) {

            $sql = "SELECT cln_codeBranch, cln_nameBranch 
            FROM b2b_client 
            WHERE cln_nit =" . $nit . " AND cln_usr_user_ID ="
                    . $idUser;
            $resultBranch = Yii::$app->db->createCommand($sql)->queryAll();
            if (count($resultBranch) > 0) {
                foreach ($resultBranch as $value) {
                    $listBranchDB[$value['cln_codeBranch']] = $value['cln_nameBranch'];
                }
            }
            //Consula a la base de datos SQL server

            $sql = "SELECT f201_id_sucursal, f201_descripcion_sucursal
                FROM t201_mm_clientes
                LEFT JOIN t200_mm_terceros AS suc ON f200_rowid = f201_rowid_tercero
                WHERE f200_nit ='" . $nit . "' AND f200_ind_estado = '1'"
                    . " ORDER BY f201_descripcion_sucursal ASC";
            $raw = Yii::$app->dbsql->createCommand($sql)->queryAll();

            if (count($raw) > 0) {
                foreach ($raw as $value) {
                    $listBranch[$value['f201_id_sucursal']] = $value['f201_descripcion_sucursal'];
                }
            } else {
                /**
                 * Si no se tienen sucursales por defecto
                 * se crea una con el nombre de la empresa
                 */
                $clientSiesa = new ClientSiesa;
                $nameClient = $clientSiesa->getNameClient($nit);
                $listBranch = array(
                    ClientSiesa::CODE_BRANCH_DEFAULT => $nameClient
                );
            }
            Yii::$app->session['lastListBranch'] = $listBranch;
        }

        return $listBranch;
    }

    /**
     * Metodo para obtener el nombre de la 
     * sucursal
     * 
     * @param string $codeBranch Codigo de la sucursal
     * 
     * @return string
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getNameBranchLastList($codeBranch) {
        $nameBranch = '';
        if (!empty($codeBranch) && is_string($codeBranch)) {
            $listBranch = Yii::$app->session['lastListBranch'];
            if (isset($listBranch[$codeBranch]))
                    $nameBranch = $listBranch[$codeBranch];
        }
        return $nameBranch;
    }

    /**
     * Metodo para obtener el nombre de la 
     * sucursal
     * 
     * @param int    $nit        Nit del cliente
     * @param string $codeBranch Codigo de la sucursal
     * 
     * @return string
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getNameBranch($nit, $codeBranch) {
        $nameBranch = '';
        if (!empty($nit) && is_numeric($nit) && !empty($codeBranch) && is_string($codeBranch)) {
            $sql = "
                SELECT 
                    f201_id_sucursal as id_sucursal, 
                    f201_descripcion_sucursal as cod_sucursal
                FROM t201_mm_clientes
                LEFT JOIN t200_mm_terceros AS suc 
                    ON f200_rowid = f201_rowid_tercero
                WHERE 
                    f200_nit ='" . $nit . "' 
                ORDER BY f201_descripcion_sucursal ASC";
            $result = Yii::$app->dbsql->createCommand($sql)->queryOne();
            if (count($result) > 0) $nameBranch = trim($result['cod_sucursal']);
        }
        return $nameBranch;
    }

    /**
     * Metodo para obtener la informacion de 
     * la sucursal 
     * 
     * @param int    $nit        Codigo del nit
     * @param string $codeBranch codigo de la sucursal
     * 
     * @return boolean
     * @throws \Exception
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public static function getInfoBranch($nit) {
        
        return ['idCo'=>'102', 'conPago'=>'90'];
        
        try {
            if (!empty($nit) && is_numeric($nit)
                    && !empty($codeBranch) && is_string($codeBranch)) {
                $sql = "SELECT f201_id_sucursal, f201_descripcion_sucursal,
                f201_id_co_factura as 'idCo', f201_id_cond_pago as 'conPago',
                f201_ind_bloqueo_cupo as 'cupo', f201_ind_bloqueo_mora as 'mora' 
                FROM t201_mm_clientes
                LEFT JOIN t200_mm_terceros AS suc ON f200_rowid = f201_rowid_tercero
                WHERE f200_nit ='" . $nit . "' AND f201_id_sucursal = '"
                        . $codeBranch . "' ORDER BY f201_descripcion_sucursal ASC";
                $result = Yii::$app->dbsql->createCommand($sql)->queryOne();
                return $result;
            }
        } catch (\Exception $exc) {
            //----------------------------------------------------
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}', 'webservices');
            //---------------------------------------------------
            return false;
        }
        return false;
    }

    /**
     * 
     * @param type $nit
     * @param type $codeBranch
     * @return boolean
     */
    public static function validateQuota($nit, $codeBranch) {
        if (!empty($nit) && is_numeric($nit)
                && !empty($codeBranch) && is_string($codeBranch)) {
            try {
                $sql = "
                    Declare @Nit_cliente char(13), @Id_Sucursal char(3), @Id_Cia int, @fecha_Menos_gracia datetime, @fecha_act datetime, @dias_gracia int, @p_cheques_posfechados money
					
                    set @Nit_cliente ='" . $nit . "'  
                    set @Id_Sucursal ='" . $codeBranch . "'
                    set @Id_Cia = 1
                    set @p_cheques_posfechados = 0
                    set @fecha_Menos_gracia = (select DATEADD(DAY, -(cli.f201_dias_gracia),GETDATE()) As FDiasGracia
                    from	t201_mm_clientes cli inner join T200_MM_TERCEROS C200 on  C200.F200_ROWID = cli.F201_ROWID_TERCERO
                    where C200.F200_ID_CIA = @Id_Cia AND C200.F200_NIT = @Nit_cliente AND cli.F201_ID_SUCURSAL = @Id_Sucursal)
                    set @fecha_act = getdate()


                    --hallar los indicadores del tercero	
                    select
                                    C200.F200_NIT Nit,
                                            C200.F200_RAZON_SOCIAL AS 'Razon Social',
                                    (case when cli.f201_ind_bloqueo_cupo = 1 then (case when (isnull(qcte.cartera_cte,0)+isnull(qcte.chq_p_Cte,0)+isnull(qvcd.chq_vencidos,0)+isnull(qvcd.Valor_vencido,0)+isnull(rem.valor_rem,0)) >=(case when not cli.f201_fecha_cupo is null and cli.f201_fecha_cupo < @fecha_act then 0 else cli.f201_cupo_credito * (1 + (cli.f201_porc_tolerancia/100)) end)then 1 else 0 end) else 0 end) as cupo,
                                    (case when cli.f201_ind_bloqueo_mora = 1 then (case when (isnull(qvcd.Valor_vencido,0)+isnull(qvcd.chq_vencidos,0)) > 0 then 1 else 0 end) else 0 end) as mora
                    from	t201_mm_clientes cli inner join T200_MM_TERCEROS C200
                    on  C200.F200_ROWID = cli.F201_ROWID_TERCERO

                    --cartera corriente en moneda local 

                    left join (Select 
                    cte.f353_rowid_tercero as tercero,
                    cte.f353_id_sucursal as sucursal,
                    isnull(sum(f353_total_db - (f353_total_cr - f353_total_ch_postf)),0) as cartera_cte,
                    isnull(sum(f353_total_ch_postf + f353_total_ch_postf_orden),0) as chq_p_Cte
                    from t353_co_saldo_abierto cte inner join t253_co_auxiliares ctass
                    on cte.f353_rowid_auxiliar = ctass.f253_rowid
                    where	f353_fecha_cancelacion is null
                    and	(f353_fecha_vcto >= @fecha_Menos_gracia) 
                    or (cte.f353_total_db < (cte.f353_total_cr - f353_total_ch_postf))
                    and	ctass.f253_ind_sa = 1
                    group by
                    cte.f353_rowid_tercero,
                    cte.f353_id_sucursal	
                    ) qcte
                    on cli.f201_rowid_tercero = qcte.tercero 
                    and cli.f201_id_sucursal = qcte.sucursal

                    --cartera vencida en moneda local 

                    left join (select	
                    vcd.f353_rowid_tercero as tercero,
                    vcd.f353_id_sucursal as sucursal,
                    isnull(sum(vcd.f353_total_db-(vcd.f353_total_cr - vcd.f353_total_ch_postf)),0) as Valor_vencido,
                    count(1) as Cantidad_vencidos,
                    isnull(sum(datediff(day,vcd.f353_fecha_vcto,@fecha_Menos_gracia)),0) as Suma_Dias_Vencidos,
                    isnull(sum(vcd.f353_total_ch_postf + vcd.f353_total_ch_postf_orden),0) as chq_vencidos
                    from t353_co_saldo_abierto vcd inner join t253_co_auxiliares ctass
                    on vcd.f353_rowid_auxiliar = ctass.f253_rowid	
                    where	vcd.f353_fecha_cancelacion is null
                    and	vcd.f353_fecha_vcto < @fecha_Menos_gracia
                    and	vcd.f353_total_db >= (vcd.f353_total_cr - vcd.f353_total_ch_postf)
                    and	ctass.f253_ind_sa = 1

                    group by
                    vcd.f353_rowid_tercero,
                    vcd.f353_id_sucursal
                    )as qvcd
                    on cli.f201_rowid_tercero = qvcd.tercero 
                    and cli.f201_id_sucursal = qvcd.sucursal

                    -- remisiones pendientes de facturar - valor neto en moneda local 

                    left join (select	
                    rm.f460_rowid_tercero_fact as terc_fac,
                    rm.f460_id_sucursal_fact as suc_fac,
                    isnull(sum(case rm.f460_ind_nat when 0 then -rm.f460_vlr_neto	else +rm.f460_vlr_neto end),0) as valor_rem,
                    count(*) as cantidad_rem
                    from	t460_cm_docto_remision_venta rm
                    where	rm.f460_ind_estado_cm in (0,1,3) --pdte. de facturar
                    and		rm.f460_cond_pago_dias_vcto <> 0
                    and		rm.f460_id_clase_docto <> 514 -- remisiones no facturables
                    and		rm.f460_id_clase_docto <> 519 -- remisiones desde pedidos no facturables
                    group by
                    rm.f460_rowid_tercero_fact,
                    rm.f460_id_sucursal_fact
                    ) as rem
                    on cli.f201_rowid_tercero = rem.terc_fac
                    and cli.f201_id_sucursal = rem.suc_fac

                    where 
                    C200.F200_ID_CIA = @Id_Cia 
                    AND C200.F200_NIT = @Nit_cliente 
                    AND cli.F201_ID_SUCURSAL = @Id_Sucursal";
                $result = Yii::$app->dbsql->createCommand($sql)->queryOne();
                return $result;
            } catch (\Exception $exc) {
                //----------------------------------------------------
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                //---------------------------------------------------
                return false;
            }
        }
        return false;
    }

    /**
     * Metodo para la consulta de productos
     * 
     * @param int    $nit   Nit del cliente  
     * @param string $cia   Codigo de la empresa
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     * 
     */
    public static function getProductos($nit, $cia, $date_start, $date_end, $percentage, $cellar_in) {
        if (!empty($nit) && !empty($cia)) {
            $sqlPrd = "
            SET NOCOUNT ON;
            DECLARE @Fecha_Final Datetime, @Nit_Cliente char(13), @Cia char(3)

                SET @Nit_cliente = '" . $nit . "'
                SET @Cia = '" . $cia . "'
                SET @Fecha_Final = CONVERT(datetime, '" . date('d-m-Y') . "', 103)  

            SELECT DISTINCT 
            'siesa' as erp,
            f120_rowid as id, 
            f126_id_lista_precio as lista_precio,
            -- f126_id_unidad_medida as unidad_medida,
            f120_id_unidad_orden as unidad_medida,
            REPLACE(LTRIM(RTRIM(f120_referencia)), ' ', '-') as 'referencia', 
            f120_descripcion as 'descripcion', 
            -- CAST(f126_precio AS INT) as 'precio_unitario', 
            -- ISNULL(CAST(f037_tasa AS INT),0) as 'iva', 
            -- CAST( ISNULL( (((f037_tasa/100)*f126_precio)+f126_precio), f126_precio) AS INT) as 'precio_unitario_iva',
            ISNULL(LTRIM(RTRIM(f106_id)),1) as 'categoria_id', 
            ISNULL(f106_descripcion, 'Todos') as 'nombre_categoria',
            ISNULL((qry_consumo.Periodo-qry_consumo.consumo)/nullif(qry_consumo.Periodo,0),0)*100 as variacion,
            ISNULL(qry_consumo.consumo,0) as  consumo,
            ISNULL(CAST(qryInv.Cantidad_Disponible AS BIGINT),0) as  'cantidad_disponible',
			ISNULL(CAST(qryReq.Cantidad AS BIGINT),0) as  'cantidad_pedida',
            f126_fecha_ts_actualizacion as fecha_actualizacion
            
            FROM t126_mc_items_precios
            JOIN t120_mc_items ON f126_rowid_item = f120_rowid
            JOIN t121_mc_items_extensiones ON f120_rowid = f121_rowid_item
            JOIN t137_mc_portafolio_items ON f137_rowid_item_ext = f121_rowid
            -- LEFT JOIN t113_mc_grupos_impositivos ON f120_id_grupo_impositivo = f113_id
            -- LEFT JOIN t114_mc_grupos_impo_impuestos ON f113_id = f114_grupo_impositivo
            -- LEFT JOIN t037_mm_llaves_impuesto ON f114_id_llave_impuesto = f037_id
            LEFT JOIN t125_mc_items_criterios ON f120_rowid = f125_rowid_item
            LEFT JOIN t106_mc_criterios_item_mayores ON f106_id_plan = f125_id_plan AND f106_id = f125_id_criterio_mayor
				
            LEFT JOIN (
                    Select 
                    MFV.f470_rowid_item_ext as row_item,
                    t200.f200_nit as tercero,
                    fv.f461_id_sucursal_fact as suc, 
                    sum(CASE WHEN MFV.F470_ID_CONCEPTO = '502' AND MFV.F470_ID_PERIODO = (YEAR(@Fecha_Final)*100+MONTH(@Fecha_Final))
                    THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END) AS Periodo,
                    sum(CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END)/6 AS Consumo
                    from
                    t470_cm_movto_invent MFV INNER JOIN t461_cm_docto_factura_venta FV
                    ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

                    INNER JOIN t200_mm_terceros T200
                    ON T200.f200_rowid = fv.f461_rowid_tercero_fact


                    group by MFV.f470_rowid_item_ext,
                    t200.f200_nit,
                    fv.f461_id_sucursal_fact) Qry_Consumo 
                    ON f121_rowid_item = qry_consumo.row_item 
                    and qry_consumo.tercero= @Nit_cliente
                    -- and qry_consumo.suc = @Suc_cliente 

			-- Reqisiciones
			
			LEFT JOIN (
					SELECT
						'Requi' as Tipo,
						T120.f120_referencia AS Item,
						T120.f120_descripcion AS Nombre_Item,
						T120.f120_id_unidad_empaque AS unidad_medida,
						MPV.f441_rowid_item_ext AS RItem,
						T150.f150_id AS Bodega,
						T150.F150_id_instalacion As Instalacion,
						SUM(MPV.f441_cant1_requerida) AS Cantidad,
						0 AS desviacion,
						0 AS promedio,
						0 AS Periodo,
						0 AS Consumo
					FROM
					t440_docto_req_int PV 
						INNER JOIN t441_movto_req_int MPV
					ON PV.f440_rowid = MPV.f441_rowid_docto_req_int

					LEFT JOIN t121_mc_items_extensiones T121
					ON T121.f121_rowid = MPV.f441_rowid_item_ext
					-- AND f121_ind_estado = 1

					LEFT JOIN t120_mc_items T120
					ON T120.f120_rowid = T121.f121_rowid_item

					LEFT JOIN t150_mc_bodegas T150
					ON MPV.f441_rowid_bodega = T150.f150_rowid

					Where
						PV.f440_id_cia = 1
						-- AND PV.f430_ind_estado NOT IN(4,9)
						AND PV.f440_id_fecha BETWEEN '$date_start' AND '$date_end'
						-- AND PV.f430_ind_facturado = 0
						-- AND PV.f430_ind_remisionado = 0
						-- AND T150.F150_ID =('$cellar_in')
					GROUP BY
						T120.f120_referencia,
						T120.f120_descripcion,
						T120.f120_id_unidad_empaque,
						MPV.f441_rowid_item_ext,
						T150.f150_id,
						T150.F150_id_instalacion
			)qryReq
			On qryReq.ritem = f121_rowid_item

			-- inventarios
            LEFT JOIN (			
                    SELECT
                       T470.F470_ROWID_ITEM_EXT AS RItem,
                       -- T470.f470_id_co_movto AS idCo,
                      Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Disponible
                    FROM T470_CM_MOVTO_INVENT T470
						  INNER JOIN t150_mc_bodegas T150
						  On T150.F150_ROWID = T470.F470_ROWID_BODEGA 
                    WHERE
                      T470.F470_ID_fecha <= @Fecha_Final 
                      AND T470.F470_IND_ESTADO_CM  <> '2' 
                      AND T150.F150_ID =('$cellar_in')
                      AND T470.F470_IND_TIPO_ITEM = 1
                    GROUP BY T470.F470_ROWID_ITEM_EXT
                    -- , T470.f470_id_co_movto
             ) qryInv	
             On qryInv.ritem = f121_rowid_item -- AND qryInv.idCo = t.idCo			


            -- LEFT JOIN (			
            --        SELECT
            --           T470.F470_ROWID_ITEM_EXT AS RItem,
            --          T470.f470_id_co_movto AS idCo,
            --          Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Disponible
            --        FROM T470_CM_MOVTO_INVENT T470 
            --        WHERE
            --          T470.F470_ID_fecha <= @Fecha_Final 
            --          AND T470.F470_IND_ESTADO_CM  <> '2' 
            --          AND T470.F470_IND_TIPO_ITEM = 1
            --        GROUP BY
            --                  T470.F470_ROWID_ITEM_EXT, T470.f470_id_co_movto
            -- ) qryInv	
            -- On qryInv.ritem = f121_rowid_item AND qryInv.idCo = t.idCo			

            WHERE 
            -- f125_id_plan = 3 
            f121_ind_estado = 1 
            -- AND f126_id_lista_precio = '001'
            AND f126_id_cia = @Cia
            AND f137_id_portafolio = '100'
            AND f137_id_cia = @Cia
            ORDER BY f120_descripcion ASC";

			\Yii::error(__FILE__ . ':' . __LINE__ . 'XXXXX{'
                        . print_r($sqlPrd, true) . '}' );
			
            $raw = Yii::$app->dbsql->createCommand($sqlPrd)->queryAll();
            if (count($raw) > 0) {
                return $raw;
            }
        }
        return array();
    }

    /**
     * Metodo para obtener el listado de categorias 
     * desde el sistema siesa, tambien permite realizar el 
     * filtro de categorias para los nit configuratos
     * 
     * @param int $nit Nit de la empresa
     * 
     * @return string
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public static function getLisCategory($nit = null) {
        $cacheCategory = Yii::$app->cache->get('listCategory');
        if ($cacheCategory === false) {

            $arrayCat = array();
            $sql = '
              SELECT LTRIM(RTRIM(f106_id)) as "f106_id", f106_descripcion
              FROM t106_mc_criterios_item_mayores
              WHERE f106_id_plan = 3
              ORDER BY f106_descripcion ASC
              ';
            $raw = Yii::$app->dbsql->createCommand($sql)->queryAll();
            if (count($raw) > 0) {
                $arrayCat = CHtml::listData($raw, 'f106_id', 'f106_descripcion');
            }

            Yii::$app->cache->set('listCategory', json_encode($arrayCat), 604800);
        } else {
            $arrayCat = json_decode($cacheCategory, true);
        }

        if (!empty($nit) && is_numeric($nit)) {
            $listCat = Yii::$app->params['nitCategoria'];
            if (count($listCat) > 0) {
                if (array_key_exists($nit, $listCat)) {
                    $arrayKeyCat = explode(',', $listCat[$nit]);
                    if (count($arrayKeyCat) > 0)
                            foreach ($arrayKeyCat as $value) {
                            if (array_key_exists($value, $arrayCat)) {
                                $arrayResult[$value] = $arrayCat[$value];
                            }
                        }
                    return $arrayResult;
                }
            }
        }
        return $arrayCat;
    }

    /**
     * Metodo para obtener la unidad de medida del item
     * 
     * @param int $reference Referencia del item
     * 
     * @return boolean
     * @throws \Exception
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getMesureUnit($references, $cia) {
        $result = [];
        try {
            if (!empty($references) && is_array($references) && !empty($cia)) {
                $sql = "
                SELECT RTRIM(LTRIM(f120_referencia)) AS f120_referencia, 
                    RTRIM(LTRIM(f120_id_unidad_orden)) as f120_id_unidad_orden
                FROM t120_mc_items 
                WHERE f120_referencia IN ('" 
                        . implode("','", $references) 
                        . "') AND f120_id_cia = '".$cia."'";
                $result = Yii::$app->dbsql->createCommand($sql)->queryAll();
            }
        } catch (\Exception $exc) {
            throw new \Exception($exc);
        }
        return $result;
    }
    
    
    /**
     * Metodo para obtener el ultimo lote de cada referencia 
     * 
     * @param array $references Referencias
     * @param int   $cia        Compania
     * @param int   $cellarIn   Bodega entrada
     * 
     * @return boolean
     * @throws \Exception
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getLastLot($references, $cia, $cellarIn) {
        $result = [];
        try {
            if (!empty($cia)) {
                $sql = "
                SELECT top 1 RTRIM(LTRIM(f401_rowid_bodega)), 
                    RTRIM(LTRIM(f120_referencia)) as f120_referencia, 
                    RTRIM(LTRIM(f401_id_lote)) AS lote 
                FROM t401_cm_existencia_lote
                JOIN t121_mc_items_extensiones ON f121_rowid = f401_rowid_item_ext
                JOIN t120_mc_items ON f120_rowid = f121_rowid_item
                JOIN t150_mc_bodegas ON f150_rowid = f401_rowid_bodega
                WHERE f120_referencia IN ('" 
                        . implode("','", $references) 
                        . "') AND f150_id = '$cellarIn'
                    AND f401_id_cia = '$cia'
                ORDER BY f401_id_lote DESC ";
                $result = Yii::$app->dbsql->createCommand($sql)->queryAll();
            }
        } catch (\Exception $exc) {
            throw new \Exception($exc);
        }
        return $result;
    }

        /**
     * Metodo para obtener el ultimo costo de los item
     * 
     * @param array $references Referencias
     * @param int   $cia        Compania
     * 
     * @return boolean
     * @throws \Exception
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getLastCost($references, $cia) {
        $result = [];
        try {
            if (!empty($cia)) {
                $sql = "  
                SELECT TOP 1
                        RTRIM(LTRIM(f120_referencia)) as f120_referencia,  
                        ISNULL(f132_ultimo_costo_local,0) AS costo,
                        f132_ts
                FROM t120_mc_items
                JOIN t121_mc_items_extensiones ON f121_rowid_item = f120_rowid
                LEFT JOIN T132_Mc_items_instalacion ON  f132_rowid_item_ext = f121_rowid
                WHERE f120_referencia IN ('" 
                        . implode("','", $references) 
                        . "') 
                  AND f132_id_cia = '$cia'
                ORDER BY f132_ts DESC";
                $result = Yii::$app->dbsql->createCommand($sql)->queryAll();
            }
        } catch (\Exception $exc) {
            throw new \Exception($exc);
        }
        return $result;
    }

    /**
     * Metodo para la consulta de los estados 
     * del pedido
     * 
     * @param string $condition Condicion 
     * 
     * @return \CArrayDataProvider
     */
    public function getOrderStatus($condition) {
        $arrayData = array();
        if (!empty($condition)) {

            try {
                $sql = "
                SELECT
                PV.f430_id_tipo_docto AS 'Tipo Documento Pedido',
                PV.f430_consec_docto AS 'pedido',
                PV.f430_id_fecha AS 'Fecha Pedido',
                PV.f430_ind_estado AS 'Estado de Pedido',
                T054.f054_descripcion AS 'estado',
                DOCFAC.f350_id_tipo_docto AS 'Tipo Documento Factura',
                DOCFAC.f350_consec_docto AS 'factura',
                DOCREM.f350_id_tipo_docto AS 'Tipo Documento Remision',
                DOCREM.f350_consec_docto AS 'remision',
                (CASE WHEN DOCFAC.f350_fecha IS NOT NULL THEN DOCFAC.f350_fecha ELSE DOCREM.f350_fecha END) AS 'fecha'
                FROM
                T430_CM_PV_DOCTO PV LEFT JOIN t460_cm_docto_remision_venta REM
                ON PV.F430_ROWID = REM.F460_ROWID_PV_DOCTO

                LEFT JOIN t461_cm_docto_factura_venta FV 
                ON REM.f460_rowid_docto_factura = FV.f461_rowid_docto

                LEFT JOIN t350_co_docto_contable DOCFAC
                ON FV.f461_rowid_docto = DOCFAC.f350_rowid

                LEFT JOIN t350_co_docto_contable DOCREM
                ON REM.f460_rowid_docto = DOCREM.f350_rowid

                LEFT JOIN T054_MM_ESTADOS T054
                ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                AND PV.f430_ind_estado = T054.F054_ID

                LEFT JOIN t200_mm_terceros T200
                ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                Where                 
                " . $condition . "
				
                ORDER BY
                PV.f430_consec_docto ASC
                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                //----------------------------------------------------
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                //---------------------------------------------------
            }
        }
        return new CArrayDataProvider($arrayData,
                array(
            //'pagination' => true,
            'keyField' => 'pedido',
            'pagination' => array(
                'pageSize' => 10,
            ),
                //'keyField'=>false
        ));
    }

    /**
     * Metodo para obtener la informacion del pedido
     * 
     * @param int $nit      Nit del cliente
     * @param int $branch   Codigo de la sucursal
     * @param int $idPedido Id del pedido
     * 
     * @return array
     */
    public function getInfoOrder($nit, $branch, $idPedido) {
        $arrayData = array();
        if (!empty($nit) && is_numeric($nit)
                && !empty($branch) && is_numeric($branch)
                && !empty($idPedido) && is_numeric($idPedido)) {
            try {
                $sql = "                              
                DECLARE @No_Pedido int, @Id_Cia int, @Nit_cliente VARCHAR(13), @Suc_cliente VARCHAR(3)

                SET @Nit_cliente ='" . $nit . "'  
                SET @Suc_cliente = '" . $branch . "'
                SET @No_pedido = '" . $idPedido . "'  
                SET @Id_Cia = 1

                SELECT distinct 
                PV.f430_id_tipo_docto AS 'Tipo Documento Pedido',
                PV.f430_consec_docto AS 'no_pedido',
                PV.f430_id_co AS 'Centro de Operacion',				
                convert(varchar, PV.f430_id_fecha, 105) AS fecha_pedido,
                PV.f430_ind_estado AS 'Estado de Pedido',
                T054.f054_descripcion AS estado,
                T200.f200_id AS nit,
                T200.F200_RAZON_SOCIAL AS nombre_cliente,
                PV.f430_id_sucursal_fact AS codigo_sucursal,
                C201.F201_DESCRIPCION_SUCURSAL AS nombre_sucursal,				
                convert(varchar, PV.f430_fecha_entrega, 105) AS fecha_entrega,
                QRY1.T_BRUTO AS total_neto,
                QRY1.T_DESC AS total_descuento,
                QRY1.T_IVA AS total_iva,
                QRY1.T_PAGAR AS total_pagar
                FROM
                T430_CM_PV_DOCTO PV 

                INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                LEFT JOIN T054_MM_ESTADOS T054
                ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                AND PV.f430_ind_estado = T054.F054_ID

                LEFT JOIN t200_mm_terceros T200
                ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                LEFT JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

                LEFT JOIN (
                SELECT
                PV1.f430_consec_docto,
                SUM(MPV.F431_VLR_BRUTO) AS T_BRUTO,
                SUM(MPV.F431_VLR_DSCTO_LINEA+MPV.F431_VLR_DSCTO_GLOBAL) AS T_DESC,
                SUM(MPV.F431_VLR_IMP) AS T_IVA,
                SUM(MPV.F431_VLR_NETO) AS T_PAGAR
                FROM
                T430_CM_PV_DOCTO PV1 INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV1.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                WHERE 
                PV1.f430_consec_docto = @No_Pedido
                AND f430_id_sucursal_rem = @Suc_cliente

                GROUP BY PV1.f430_consec_docto) AS QRY1
                ON PV.f430_consec_docto = QRY1.f430_consec_docto

                Where 
                PV.f430_id_cia = @Id_Cia 
                AND PV.f430_consec_docto = @No_Pedido
                AND t200.f200_id  = @Nit_cliente
                AND C201.f201_id_sucursal = @Suc_cliente
                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryOne();
            } catch (\Exception $exc) {
                //----------------------------------------------------
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                //---------------------------------------------------
            }
            return $arrayData;
        }
    }

    /**
     * Metodo para obtener la informacion detallada por item del pedido
     * 
     * @param int $nit      Nit del cliente
     * @param int $branch   Codigo de la sucursal
     * @param int $idPedido Id del pedido
     * 
     * @return array
     */
    public function getDetialItemsOrder($nit, $branch, $idPedido) {
        $arrayData = array();
        if (!empty($nit) && is_numeric($nit)
                && !empty($branch) && is_numeric($branch)
                && !empty($idPedido) && is_numeric($idPedido)) {
            try {
                $sql = "                              
                DECLARE @No_Pedido int, @Id_Cia int, @Nit_cliente VARCHAR(13), @Suc_cliente VARCHAR(3)

                SET @Nit_cliente ='" . $nit . "'  
                SET @Suc_cliente = '" . $branch . "'
                SET @No_pedido = '" . $idPedido . "'  
                SET @Id_Cia = 1

                SELECT
                T120.f120_referencia AS referencia,
                T120.f120_descripcion AS descripcion,
                PV1.f430_consec_docto,
                f430_id_co,
                f430_id_sucursal_rem,
                MPV.F431_CANT1_REMISIONADA AS cantidad,
                MPV.F431_PRECIO_UNITARIO_BASE AS precio_unitario,
                MPV.F431_VLR_BRUTO AS subtotal_unitario,
                MPV.F431_VLR_IMP AS iva,
                MPV.F431_PRECIO_UNITARIO_BASE + MPV.F431_VLR_IMP as precio_unitario_iva,
                MPV.F431_VLR_NETO AS subtotal_iva
                FROM
                T430_CM_PV_DOCTO PV1 INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV1.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                LEFT JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

                WHERE 
                PV1.f430_consec_docto = @No_Pedido
                AND f430_id_sucursal_rem = @Suc_cliente
                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
            return $arrayData;
        }
    }

    /**
     * Retorna array para el grid para las fac y rem
     * 
     * @param int $Pedido // condicion del array
     * @return array
     * 
     * @author Duvan Agudelo <John.agudelo@ingeneo.com.co>
     * @copyright 2015 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getDetialOrderFactureRemi($Pedido) {
        $arrayData = array();
        if (!empty($Pedido) && is_numeric($Pedido)) {
            try {
                $sqlremi = "                              
                DECLARE @Id_Cia int, @N_Pedido int

                SET @Id_Cia = 1
                SET @N_Pedido = '" . $Pedido . "'

                SELECT
                T120.f120_id AS Item,
                T120.f120_referencia AS Referencia,
                T120.f120_descripcion AS 'Nombre_Item',
                T200.f200_id AS 'Nit_Cliente',
                T200.F200_RAZON_SOCIAL AS 'Nombre_Cliente',
                PV.f430_id_sucursal_fact AS 'Sucursal_Factura',
                PV.f430_id_sucursal_rem AS 'Sucursal_Remision',
                C201.F201_DESCRIPCION_SUCURSAL AS 'Nombre_Sucursal',
                'Remisionado' as Tipo,
                PV.f430_id_tipo_docto AS 'Tipo_Documento_Pedido',
                PV.f430_consec_docto AS 'Numero_Pedido',
                DOCREM.f350_id_tipo_docto AS 'Tipo_Documento',
                DOCREM.f350_consec_docto AS 'Numero_Documento',
                MPV.F431_CANT1_REMISIONADA AS 'Cantidad',
                MPV.F431_PRECIO_UNITARIO_BASE AS 'Precio_Unitario',
                MPV.F431_VLR_BRUTO AS 'Subtotal_Unitario',
                MPV.F431_VLR_IMP AS 'IVA',
                MPV.F431_VLR_NETO AS 'Subtotal_con_IVA'
                FROM
                T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                LEFT JOIN t460_cm_docto_remision_venta REM
                ON PV.F430_ROWID = REM.F460_ROWID_PV_DOCTO

                LEFT JOIN t461_cm_docto_factura_venta FV 
                ON REM.f460_rowid_docto_factura = FV.f461_rowid_docto

                LEFT JOIN t350_co_docto_contable DOCREM
                ON REM.f460_rowid_docto = DOCREM.f350_rowid

                LEFT JOIN T054_MM_ESTADOS T054
                ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                AND PV.f430_ind_estado = T054.F054_ID

                INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                LEFT JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

                Where 
                PV.f430_consec_docto = @N_Pedido					
                ";

                $sqlfac = "                              
                DECLARE @Id_Cia int, @N_Pedido int

                SET @Id_Cia = 1
                SET @N_Pedido = '" . $Pedido . "'

                SELECT
                T120.f120_id AS Item,
                T120.f120_referencia AS Referencia,
                T120.f120_descripcion AS 'Nombre_Item',
                T200.f200_id AS 'Nit_Cliente',
                T200.F200_RAZON_SOCIAL AS 'Nombre_Cliente',
                PV.f430_id_sucursal_fact AS 'Sucursal_Factura',
                PV.f430_id_sucursal_rem AS 'Sucursal_Remision',
                C201.F201_DESCRIPCION_SUCURSAL AS 'Nombre_Sucursal',
                'Facturado' as Tipo,
                PV.f430_id_tipo_docto AS 'Tipo_Documento_Pedido',
                PV.f430_consec_docto AS 'Numero_Pedido',
                DOCFAC.f350_id_tipo_docto AS 'Tipo_Documento',
                DOCFAC.f350_consec_docto AS 'Documento',
                MPV.F431_CANT1_REMISIONADA AS 'Cantidad',
                MPV.F431_PRECIO_UNITARIO_BASE AS 'Precio_Unitario',
                MPV.F431_VLR_BRUTO AS 'Subtotal_Unitario',
                MPV.F431_VLR_IMP AS 'IVA',
                MPV.F431_VLR_NETO AS 'Subtotal_con_IVA'
                FROM
                T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                LEFT JOIN t460_cm_docto_remision_venta REM
                ON PV.F430_ROWID = REM.F460_ROWID_PV_DOCTO

                LEFT JOIN t461_cm_docto_factura_venta FV 
                ON REM.f460_rowid_docto_factura = FV.f461_rowid_docto

                LEFT JOIN t350_co_docto_contable DOCFAC
                ON FV.f461_rowid_docto = DOCFAC.f350_rowid

                LEFT JOIN T054_MM_ESTADOS T054
                ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                AND PV.f430_ind_estado = T054.F054_ID

                INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

                LEFT JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

                Where 
                PV.f430_consec_docto = @N_Pedido					
                ";
                $arrayRemi = Yii::$app->dbsql->createCommand($sqlremi)->queryAll();
                $arrayFac = Yii::$app->dbsql->createCommand($sqlfac)->queryAll();

                $arrayData = array("remi" => $arrayRemi, "fac" => $arrayFac);
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
            return $arrayData;
        }
    }

    /**
     * Retorna dataprovider para el grid
     * 
     * @param string $condicion // condicion del array
     * @return Dataprovider
     * 
     * @author Duvan Agudelo <John.agudelo@ingeneo.com.co>
     * @copyright 2015 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getOrderBudget($condicion) {
        $arrayData = array();


        if (!empty($condicion) && is_string($condicion)) {
            try {
                $sql = "                              
                    DECLARE @Id_Cia int,@Fecha_inicial Datetime, @Fecha_Final Datetime, @Nit_Cliente char (13), @Suc_Cliente char (3)

                    SET @Id_Cia = 1
                    SET @Fecha_Inicial = '01/10/2015' 
                    SET @Fecha_Final = '25/11/2015' 
                    SET @Nit_Cliente = '890900842'
                    SET @Suc_Cliente = '152'

                    SELECT
                    T200.f200_id AS 'Nit Cliente',
                    T200.F200_RAZON_SOCIAL AS 'Nombre Cliente',
                    PV.f430_id_sucursal_fact AS 'Sucursal Factura',
                    PV.f430_id_sucursal_rem AS 'Sucursal Remision',
                    C201.F201_DESCRIPCION_SUCURSAL AS 'Nombre Sucursal Factura',
                    C201C.F201_DESCRIPCION_SUCURSAL AS 'Nombre Sucursal Remision',
                    sum(CASE WHEN PV.F430_IND_FACTURADO  = 1 THEN MPV.F431_VLR_NETO ELSE 0 END)as Valor_Facturado,
                    sum(CASE WHEN PV.F430_IND_FACTURADO  = 0 THEN MPV.F431_VLR_NETO ELSE 0 END)as Valor_Remisionado,
                    sum(MPV.F431_VLR_NETO) AS 'Subtotal con Iva',
                    sum(MPV.F431_VLR_BRUTO) AS 'Subtotal Sin Iva',
                    sum(MPV.F431_CANT1_REMISIONADA) AS 'Cantidad'
                    FROM
                    T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
                    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                    LEFT JOIN T054_MM_ESTADOS T054
                    ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                    AND PV.f430_ind_estado = T054.F054_ID

                    INNER JOIN t200_mm_terceros T200
                    ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                    INNER JOIN T201_MM_CLIENTES C201
                    ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                    AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                    INNER JOIN T201_MM_CLIENTES C201C
                    ON T200.F200_ROWID = C201C.F201_ROWID_TERCERO
                    AND  PV.F430_ID_SUCURSAL_REM = C201C.F201_ID_SUCURSAL

                    LEFT JOIN t121_mc_items_extensiones T121
                    ON T121.f121_rowid = MPV.f431_rowid_item_ext

                    LEFT JOIN t120_mc_items T120
                    ON T120.f120_rowid = T121.f121_rowid_item

                    Where 
                    PV.f430_id_cia = @Id_Cia 
                    AND PV.f430_ind_estado NOT IN(0,9)
                    AND PV.F430_IND_REMISIONADO = 1
                   " . $condicion . "

                    Group by 
                    T200.f200_id,
                    T200.F200_RAZON_SOCIAL,
                    PV.f430_id_sucursal_fact,
                    PV.f430_id_sucursal_rem,
                    C201.F201_DESCRIPCION_SUCURSAL, 
                    C201C.F201_DESCRIPCION_SUCURSAL 


                    Order By T200.f200_id ,
                    PV.f430_id_sucursal_fact,
                    PV.f430_id_sucursal_rem				
                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        $dataProvider = new CArrayDataProvider($arrayData,
                array(
            'keyField' => 'Nit Cliente',
            'pagination' => array(
                'pageSize' => 10,
            ),
        ));

        return $dataProvider;
    }

    /**
     * Metodo para obtener la informacion detallada por item del pedido
     * 
     * @param int $nit      Nit del cliente
     * @param int $branch   Codigo de la sucursal
     * @param int $idPedido Id del pedido
     * 
     * @return array
     */
    public function getInfoBill($idFactura) {
        $arrayData = array();
        if (!empty($idFactura) && is_numeric($idFactura)) {
            try {
                $sql = "                              
                Declare @No_Factura char(9), @Id_Cia int
				set @No_Factura ='" . $idFactura . "'
				set @Id_Cia = 1

				select 
				DOCFAC.f350_id_tipo_docto AS 'Tipo Documento',
				DOCFAC.f350_consec_docto AS no_factura,
				CLIENTE.f200_id AS nit,
				CLIENTE.F200_RAZON_SOCIAL AS nombre_cliente,
				FV.F461_ID_SUCURSAL_FACT AS codigo_sucursal,
				C201.F201_DESCRIPCION_SUCURSAL AS nombre_sucursal,
				C015.F015_DIRECCION1 AS Direccion,
				C013.f013_descripcion AS Ciudad,
				VENDEDOR.f200_id AS 'NIT Vendedor',
				VENDEDOR.f200_razon_social AS 'Nombre Vendedor',
				PV.F430_ID_TIPO_DOCTO AS 'Tipo Documento Pedido',
				PV.F430_CONSEC_DOCTO AS numero_pedido,
				PV.F430_ID_FECHA AS 'Fecha Pedido',
				convert(varchar, DOCFAC.f350_fecha, 105) AS fecha_factura,			
				CP.F208_DESCRIPCION AS forma_pago,
				R015.F015_DIRECCION1 AS direccio_envio,
				R013.f013_descripcion AS ciudad_envio,
				FV.F461_VLR_BRUTO AS total_neto,
				FV.F461_VLR_DSCTO AS total_descuento,
				FV.F461_VLR_BRUTO AS subtotal,
				FV.F461_VLR_IMP AS total_iva,
				FV.F461_VLR_NETO AS total_pagar
				FROM 
				t461_cm_docto_factura_venta FV

				INNER JOIN t350_co_docto_contable DOCFAC
				ON DOCFAC.f350_rowid = FV.f461_rowid_docto

				LEFT JOIN t460_cm_docto_remision_venta T460
				ON T460.f460_rowid_docto_factura = FV.f461_rowid_docto

				LEFT JOIN T430_CM_PV_DOCTO PV
				ON T460.F460_ROWID_PV_DOCTO = PV.F430_ROWID

				LEFT JOIN T208_MM_CONDICIONES_PAGO CP
				ON FV.F461_ID_COND_PAGO = CP.F208_ID
				AND FV.F461_ID_CIA = CP.F208_ID_CIA

				LEFT JOIN t200_mm_terceros CLIENTE
				ON CLIENTE.f200_rowid = FV.f461_rowid_tercero_fact

				INNER JOIN T201_MM_CLIENTES C201
				ON CLIENTE.F200_ROWID = C201.F201_ROWID_TERCERO
				AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

				LEFT JOIN t200_mm_terceros VENDEDOR
				ON VENDEDOR.f200_rowid = FV.f461_rowid_tercero_vendedor

				LEFT JOIN T215_MM_PUNTOS_ENVIO_CLIENTE T215
				ON T460.F460_ROWID_PUNTO_ENVIO_REM = T215.F215_ROWID

				Left Join t015_mm_contactos C015
				ON cliente.f200_rowid_contacto = C015.f015_rowid

				Left Join t015_mm_contactos R015
				ON T215.F215_ROWID_CONTACTO = R015.f015_rowid
				
				Left Join t011_mm_paises C011 
				ON C015.f015_id_pais = C011.f011_id

				Left Join t012_mm_deptos C012 
				ON C015.f015_id_depto = C012.f012_id 
				AND C012.f012_id_pais = C011.f011_id

				Left Join t013_mm_ciudades C013 
				ON C015.f015_id_ciudad = C013.f013_id 
				AND C013.f013_id_depto = C012.f012_id 
				AND C013.f013_id_pais = C011.f011_id

				Left Join t011_mm_paises R011 
				ON R015.f015_id_pais = R011.f011_id

				Left Join t012_mm_deptos R012 
				ON R015.f015_id_depto = R012.f012_id 
				AND R012.f012_id_pais = R011.f011_id

				Left Join t013_mm_ciudades R013 
				ON R015.f015_id_ciudad = R013.f013_id 
				AND R013.f013_id_depto = R012.f012_id 
				AND R013.f013_id_pais = R011.f011_id

				WHERE
				FV.f461_id_cia = @Id_Cia 
				AND DOCFAC.f350_id_tipo_docto = 'FV'
				AND DOCFAC.f350_consec_docto = @No_Factura
                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryOne();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
            return $arrayData;
        }
    }

    /**
     * Metodo para obtener la informacion detallada por item del pedido
     * 
     * @param int $nit      Nit del cliente
     * @param int $branch   Codigo de la sucursal
     * @param int $idPedido Id del pedido
     * 
     * @return array
     */
    public function getDetialItemsBill($idFactura) {
        $arrayData = array();
        if (!empty($idFactura) && is_numeric($idFactura)) {
            try {
                $sql = "                              
				Declare @No_Factura char(9), @Id_Cia int
				set @No_Factura ='" . $idFactura . "'
				set @Id_Cia = 1

				select 
				T120.f120_id AS Item,
				T120.f120_referencia AS referencia,
				T120.f120_descripcion AS descripcion,
				(CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END) AS cantidad,
				MFV.f470_precio_uni AS precio_unitario,
				(CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END) AS subtotal_unitario,
				(CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_vlr_imp*-1 ELSE MFV.f470_vlr_imp END) AS iva,
				(CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto-MFV.f470_vlr_dscto_linea+MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_bruto-MFV.f470_vlr_dscto_linea+MFV.f470_vlr_imp END) AS subtotal_iva
				FROM 
				t470_cm_movto_invent MFV INNER JOIN t461_cm_docto_factura_venta FV
				ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

				INNER JOIN t350_co_docto_contable DOCFAC
				ON DOCFAC.f350_rowid = FV.f461_rowid_docto

				LEFT JOIN t121_mc_items_extensiones T121
				ON T121.f121_rowid = MFV.f470_rowid_item_ext

				LEFT JOIN t120_mc_items T120
				ON T120.f120_rowid = T121.f121_rowid_item

				WHERE
				FV.f461_id_cia = @Id_Cia 
				AND DOCFAC.f350_id_tipo_docto = 'FV'
				AND DOCFAC.f350_consec_docto = @No_Factura

                ";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
            return $arrayData;
        }
    }

    /**
     * Metodo para consulta la informacion del portafolio
     * 
     * @param string $endDate    Fecha final
     * @param int    $codClient  Nit del cliente
     * @param int    $codeBranch Codigo de la sucursal
     * 
     * @return array
     */
    public function getInfoPortfolio($endDate, $codClient, $codeBranch) {
        $arrayData = array();
        if (!empty($endDate) && is_string($endDate)
                && !empty($codClient) && is_string($codClient)
                && !empty($codeBranch) && is_string($codeBranch)) {
            try {
                $sql = "
                DECLARE @Nit_cliente VARCHAR(13), @Suc_cliente VARCHAR(3), @fecha_dto_fin DATETIME, @Id_Cia INT

				SET @Nit_cliente ='" . $codClient . "'  
				SET @Suc_cliente = '" . $codeBranch . "'
				SET @Id_Cia = 1
				SET @fecha_dto_fin = Convert(DATETIME,'" . $endDate . "',101)

				SELECT Nit_Cliente, Sucursal_cliente, Cupo_Credito AS cupo_total, Saldo_Cliente as cxp, Cupo_Credito-Saldo_Cliente as cupo_disponible
				FROM(
				SELECT 
				t200.f200_id AS Nit_Cliente,
				t351.f351_id_sucursal AS Sucursal_cliente,
				t201.f201_cupo_credito AS Cupo_Credito,
				SUM(ISNULL(t354.f354_valor_db,t351.f351_valor_db)-ISNULL(t354.f354_valor_cr,t351.f351_valor_cr)) AS Saldo_Cliente
				FROM 
				T350_CO_DOCTO_CONTABLE T350 Inner Join T351_CO_MOV_DOCTO T351 
				On T350.F350_ROWID = T351.F351_ROWID_DOCTO 

				Inner Join T253_CO_AUXILIARES T253 
				On T351.F351_ROWID_AUXILIAR = T253.F253_ROWID  

				Inner Join T254_CO_MAYORES_AUXILIARES T254 
				On T253.F253_ROWID = T254.F254_ROWID_AUXILIAR  

				Inner Join T252_CO_MAYORES T252 
				On T252.F252_ID = T254.F254_ID_MAYOR And
				T252.F252_ID_PLAN = T254.F254_ID_PLAN 

				Left Join t354_co_mov_saldo_abierto t354 
				ON t351.f351_rowid  = t354.f354_rowid_mov_docto

				Left Join t353_co_saldo_abierto t353 
				ON t354.f354_rowid_sa = t353.f353_rowid 
				AND t354.f354_ind_estado = 1 

				Left Join T200_MM_TERCEROS T200 
				On T351.F351_ROWID_TERCERO = T200.F200_ROWID

				Inner Join t201_mm_clientes t201 
				ON t201.f201_rowid_tercero = t200.f200_rowid 
				AND t201.f201_id_sucursal =t351.f351_id_sucursal

				WHERE  
				T350.F350_IND_ESTADO = '1' 
				AND  T351.F351_ID_CIA = @Id_Cia 
				AND ( LEFT(T252.F252_id,4) IN ('1305','1325','1365','1380','1390','2605','2805','2810') )
				AND ( RTRIM(T252.F252_id) NOT IN ('260525','260530','260535','260595','261025','261095') )
				AND (ISNULL(t353.f353_fecha_cancelacion,@fecha_dto_fin  + 1)> @fecha_dto_fin ) 
				AND (ISNULL(t350.f350_fecha,@fecha_dto_fin )<= @fecha_dto_fin )
				AND t200.f200_id  = @Nit_cliente
				AND t351.f351_id_sucursal = @Suc_cliente
				GROUP BY 
				t200.f200_id,
				t351.f351_id_sucursal,
				t201.f201_cupo_credito) query
				";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryOne();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        return $arrayData;
    }

    /**
     * Metodo para consulta el listato de las facturas
     * 
     * @param string $endDate    Fecha final
     * @param int    $codClient  Nit del cliente
     * @param int    $codeBranch Codigo de la sucursal
     * 
     * @return array
     */
    public function getListBillPendig($endDate, $codClient, $codeBranch) {
        $arrayData = array();
        if (!empty($endDate) && is_string($endDate)
                && !empty($codClient) && is_string($codClient)
                && !empty($codeBranch) && is_string($codeBranch)) {
            try {
                $sql = "
                DECLARE @Nit_cliente VARCHAR(13), @Suc_cliente VARCHAR(3), @fecha_dto_fin DATETIME, @Id_Cia INT

				SET @Nit_cliente ='" . $codClient . "'  
				SET @Suc_cliente = '" . $codeBranch . "'
				SET @Id_Cia = 1
				SET @fecha_dto_fin = Convert(DATETIME,'" . $endDate . "',101)

				SELECT 
				t200.f200_id AS 'Nit Cliente',
				t200.f200_razon_social AS 'Nombre Cliente',
				t351.f351_id_sucursal AS Sucursal_cliente,
				LTRIM(RTRIM(t353.f353_id_tipo_docto_cruce)) AS tipo_doc,
				t353.f353_consec_docto_cruce AS no_factura,
				convert(varchar, t353.f353_fecha_vcto, 105) AS fecha_factura,
				DATEDIFF(DAY,t353.f353_fecha_vcto,@fecha_dto_fin) AS dias_vencimiento,
				DATEDIFF(DAY,t353.f353_fecha_vcto,getdate()) AS dias_vencidos,
				SUM(ISNULL(t354.f354_valor_db,t351.f351_valor_db)-ISNULL(t354.f354_valor_cr,t351.f351_valor_cr)) AS valor_pendiente
				FROM 
				T350_CO_DOCTO_CONTABLE T350 Inner Join T351_CO_MOV_DOCTO T351 
				On T350.F350_ROWID = T351.F351_ROWID_DOCTO 

				Inner Join T253_CO_AUXILIARES T253 
				On T351.F351_ROWID_AUXILIAR = T253.F253_ROWID  

				Inner Join T254_CO_MAYORES_AUXILIARES T254 
				On T253.F253_ROWID = T254.F254_ROWID_AUXILIAR  

				Inner Join T252_CO_MAYORES T252 
				On T252.F252_ID = T254.F254_ID_MAYOR And
				T252.F252_ID_PLAN = T254.F254_ID_PLAN 

				Left Join t354_co_mov_saldo_abierto t354 
				ON t351.f351_rowid  = t354.f354_rowid_mov_docto

				Left Join t353_co_saldo_abierto t353 
				ON t354.f354_rowid_sa = t353.f353_rowid 
				AND t354.f354_ind_estado = 1 

				Left Join T200_MM_TERCEROS T200 
				On T351.F351_ROWID_TERCERO = T200.F200_ROWID

				Inner Join t201_mm_clientes t201 
				ON t201.f201_rowid_tercero = t200.f200_rowid 
				AND t201.f201_id_sucursal =t351.f351_id_sucursal 

				WHERE  
				T350.F350_IND_ESTADO = '1' 
				AND  T351.F351_ID_CIA = 1 
				AND ( LEFT(T252.F252_id,4) IN ('1305','1325','1365','1380','1390','2605','2805','2810') )
				AND ( RTRIM(T252.F252_id) NOT IN ('260525','260530','260535','260595','261025','261095') )
				AND (ISNULL(t353.f353_fecha_cancelacion,@fecha_dto_fin  + 1)> @fecha_dto_fin ) 
				AND (ISNULL(t350.f350_fecha,@fecha_dto_fin )<= @fecha_dto_fin )
				AND t200.f200_id  = @Nit_cliente
				AND t351.f351_id_sucursal = @Suc_cliente

				GROUP BY t200.f200_id, 
				t200.f200_razon_social, 
				t351.f351_id_sucursal, 
				t201.f201_cupo_credito,
				t353.f353_id_tipo_docto_cruce, 
				t353.f353_consec_docto_cruce, 
				t353.f353_fecha_vcto
				
				ORDER BY
				fecha_factura ASC
				";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        return $arrayData;
    }

    /**
     * Metodo para consulta del nivel de servicio
     * 
     * @param string $startDate  Fecha Inicial
     * @param string $endDate    Fecha final
     * @param int    $codClient  Nit del cliente
     * @param int    $codeBranch Codigo de la sucursal
     * 
     * @return array
     */
    public function getInfoLevelService($startDate, $endDate, $codClient,
            $codeBranch) {
        $arrayData = array();
        if (!empty($startDate) && is_string($startDate)
                && !empty($endDate) && is_string($endDate)
                && !empty($codClient) && is_string($codClient)
                && !empty($codeBranch) && is_string($codeBranch)) {
            try {
                $sql = "
                DECLARE @Nit_cliente char(13), @C_Sucursal varchar(3), @fecha_dto_ini DATETIME, @fecha_dto_fin DATETIME, @Id_Cia int
				SET @Nit_cliente ='" . $codClient . "'  
				SET @C_Sucursal = '" . $codeBranch . "'
				SET @fecha_dto_ini = Convert(DATETIME,'" . $startDate . "',101)
				SET @fecha_dto_fin = Convert(DATETIME,'" . $endDate . "',101)
				SET @Id_Cia = 1

				SELECT
				T120.f120_id AS Item,
				T120.f120_referencia AS referencia,
				T120.f120_descripcion AS descripcion,
				T106.f106_id AS 'id_Categoria',
				T106.f106_descripcion AS categoria,
				SUM(MPV.F431_CANT1_PEDIDA) AS cantidad_pedida,
				SUM(MPV.F431_CANT1_FACTURADA) AS cantidad_facturada,
				(CASE WHEN MPV.F431_CANT1_PEDIDA = 0 THEN 0 ELSE SUM(MPV.F431_CANT1_FACTURADA * 100)/SUM(MPV.F431_CANT1_PEDIDA) END) AS n_servicio,
				SUM(MPV.F431_CANT1_PEDIDA* MPV.F431_PRECIO_UNITARIO_BASE) AS total_pedido,
				SUM(MPV.F431_CANT1_FACTURADA* MPV.F431_PRECIO_UNITARIO_BASE) AS total_facturado,
				(CASE WHEN SUM(MPV.F431_CANT1_PEDIDA* MPV.F431_PRECIO_UNITARIO_BASE) = 0 THEN 0 ELSE SUM(MPV.F431_CANT1_FACTURADA* MPV.F431_PRECIO_UNITARIO_BASE) * 100/SUM(MPV.F431_CANT1_PEDIDA* MPV.F431_PRECIO_UNITARIO_BASE) END) AS n_servicio_facturado
				FROM
				T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
				ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

				LEFT JOIN t200_mm_terceros T200
				ON T200.f200_rowid = PV.f430_rowid_tercero_fact

				LEFT JOIN t121_mc_items_extensiones T121
				ON T121.f121_rowid = MPV.f431_rowid_item_ext

				LEFT JOIN t120_mc_items T120
				ON T120.f120_rowid = T121.f121_rowid_item

				LEFT JOIN t125_mc_items_criterios T125
				ON T120.f120_rowid = T125.f125_rowid_item

				LEFT JOIN t106_mc_criterios_item_mayores T106
				ON T106.f106_id_plan = T125.f125_id_plan 
				AND T106.f106_id = T125.f125_id_criterio_mayor

				LEFT JOIN T149_Mc_Tipo_Inv_Serv Tpiv
				ON T120.f120_id_tipo_inv_serv = Tpiv.F149_ID

				WHERE
				PV.f430_id_cia = @Id_Cia 
				AND t200.f200_id = @Nit_cliente
				AND PV.F430_ID_SUCURSAL_FACT = @C_Sucursal
				AND PV.f430_id_fecha  BETWEEN @fecha_dto_ini AND @fecha_dto_fin
				AND T125.f125_id_plan = 3
				GROUP BY
				T120.f120_id,
				T120.f120_referencia,
				T120.f120_descripcion,
				T106.f106_id,
				T106.f106_descripcion,
				MPV.F431_CANT1_PEDIDA
				
				ORDER BY 
				T120.f120_descripcion ASC
				";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        return $arrayData;
    }

    /**
     * Retorna dataprovider para el grid
     * 
     * @param String $condition // condición para el array
     * @return \CArrayDataProvider
     * 
     * @author Duvan Agudelo <John.agudelo@ingeneo.com.co>
     * @copyright 2015 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getOrderfacture($condition) {

        $arrayData = array();
        if (!empty($condition) && is_string($condition)) {
            try {

                $sql = "DECLARE @Id_Cia int

                    SET @Id_Cia = 1
                     

                    SELECT
                    PV.f430_consec_docto AS id,    
                    PV.f430_consec_docto AS 'Pedido',
                    T200.F200_RAZON_SOCIAL AS 'Cliente',
                    C201.F201_DESCRIPCION_SUCURSAL AS 'Sucursal',
                    'Remisionado' as Tipo

                    FROM
                    T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
                    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                    LEFT JOIN t460_cm_docto_remision_venta REM
                    ON PV.F430_ROWID = REM.F460_ROWID_PV_DOCTO

                    LEFT JOIN t461_cm_docto_factura_venta FV 
                    ON REM.f460_rowid_docto_factura = FV.f461_rowid_docto

                    LEFT JOIN t350_co_docto_contable DOCREM
                    ON REM.f460_rowid_docto = DOCREM.f350_rowid

                    LEFT JOIN T054_MM_ESTADOS T054
                    ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                    AND PV.f430_ind_estado = T054.F054_ID

                    INNER JOIN t200_mm_terceros T200
                    ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                    INNER JOIN T201_MM_CLIENTES C201
                    ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                    AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                    LEFT JOIN t121_mc_items_extensiones T121
                    ON T121.f121_rowid = MPV.f431_rowid_item_ext

                    LEFT JOIN t120_mc_items T120
                    ON T120.f120_rowid = T121.f121_rowid_item

                    Where 
                    PV.f430_id_cia = @Id_Cia 
                    AND PV.F430_IND_REMISIONADO = 1
                    AND PV.F430_IND_FACTURADO = 0
                    AND " . $condition . "
                    group by PV.f430_consec_docto,T200.F200_RAZON_SOCIAL,C201.F201_DESCRIPCION_SUCURSAL

                    union all

                    SELECT
                    PV.f430_consec_docto AS id,
                    PV.f430_consec_docto AS 'Pedido',
                    T200.F200_RAZON_SOCIAL AS 'Cliente',
                    C201.F201_DESCRIPCION_SUCURSAL AS 'Sucursal',
                    'Facturado' as Tipo
                    FROM
                    T430_CM_PV_DOCTO PV INNER JOIN T431_CM_PV_MOVTO MPV
                    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                    LEFT JOIN t460_cm_docto_remision_venta REM
                    ON PV.F430_ROWID = REM.F460_ROWID_PV_DOCTO

                    LEFT JOIN t461_cm_docto_factura_venta FV 
                    ON REM.f460_rowid_docto_factura = FV.f461_rowid_docto

                    LEFT JOIN t350_co_docto_contable DOCFAC
                    ON FV.f461_rowid_docto = DOCFAC.f350_rowid

                    LEFT JOIN T054_MM_ESTADOS T054
                    ON PV.F430_ID_GRUPO_CLASE_DOCTO = T054.F054_ID_GRUPO_CLASE_DOCTO
                    AND PV.f430_ind_estado = T054.F054_ID

                    INNER JOIN t200_mm_terceros T200
                    ON T200.f200_rowid = PV.f430_rowid_tercero_fact

                    INNER JOIN T201_MM_CLIENTES C201
                    ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                    AND PV.F430_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL

                    LEFT JOIN t121_mc_items_extensiones T121
                    ON T121.f121_rowid = MPV.f431_rowid_item_ext

                    LEFT JOIN t120_mc_items T120
                    ON T120.f120_rowid = T121.f121_rowid_item

                    Where 
                    PV.f430_id_cia = @Id_Cia 
                    AND PV.F430_IND_FACTURADO = 1
                    AND " . $condition . "		
                    group by PV.f430_consec_docto,T200.F200_RAZON_SOCIAL,C201.F201_DESCRIPCION_SUCURSAL ";

                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        $dataProvider = new CArrayDataProvider($arrayData,
                array(
            'keyField' => 'id',
            'id' => 'id',
            'pagination' => array(
                'pageSize' => 10,
            ),
        ));

        return $dataProvider;
    }

    /**
     * Metodo para consulta del nivel de servicio
     * 
     * @param string $startDate  Fecha Inicial
     * @param string $endDate    Fecha final
     * @param int    $codClient  Nit del cliente
     * @param int    $codeBranch Codigo de la sucursal
     * 
     * @return array
     */
    public function getInfoSales($startDate, $endDate, $codClient, $codeBranch) {
        $arrayData = array();
        if (!empty($startDate) && is_string($startDate)
                && !empty($endDate) && is_string($endDate)
                && !empty($codClient) && is_string($codClient)
                && !empty($codeBranch) && is_string($codeBranch)) {
            try {
                $sql = "
                DECLARE @Nit_cliente char(13), @C_Sucursal varchar(3), @fecha_dto_ini DATETIME, @fecha_dto_fin DATETIME, @Id_Cia int
				SET @Nit_cliente ='" . $codClient . "'  
				SET @C_Sucursal = '" . $codeBranch . "'
				SET @fecha_dto_ini = Convert(DATETIME,'" . $startDate . "',101)
				SET @fecha_dto_fin = Convert(DATETIME,'" . $endDate . "',101)
				SET @Id_Cia = 1
				
				select				
				T120.f120_referencia AS referencia,
				T120.f120_descripcion AS descripcion,
				MFV.f470_precio_uni AS precio_unitario,
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END)) AS total_cantidad_pedida,
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS sub_cantidad_pedida
				FROM 
				t470_cm_movto_invent MFV INNER JOIN t461_cm_docto_factura_venta FV
				ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

				INNER JOIN t350_co_docto_contable DOCFAC
				ON DOCFAC.f350_rowid = FV.f461_rowid_docto

				LEFT JOIN t121_mc_items_extensiones T121
				ON T121.f121_rowid = MFV.f470_rowid_item_ext

				LEFT JOIN t120_mc_items T120
				ON T120.f120_rowid = T121.f121_rowid_item

				LEFT JOIN t200_mm_terceros CLIENTE
				ON CLIENTE.f200_rowid = FV.f461_rowid_tercero_fact

				WHERE
				FV.f461_id_cia = @Id_Cia 
				AND CLIENTE.f200_id = @Nit_cliente
				AND FV.F461_ID_SUCURSAL_FACT = @C_Sucursal
				AND DOCFAC.f350_fecha BETWEEN @fecha_dto_ini AND @fecha_dto_fin
				AND DOCFAC.f350_id_tipo_docto = 'FV'

				GROUP BY
				T120.f120_id,
				T120.f120_referencia,
				T120.f120_descripcion,
				MFV.f470_precio_uni
				
				ORDER BY 
				T120.f120_descripcion ASC	
				";
                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        return $arrayData;
    }

    /**
     * Metodo que retorna el tipo de documento
     * 
     * @param string $type Tipo de documento
     * 
     * @return string
     */
    public static function getTypeDocument($type) {
        switch ($type) {
            case 'FV':
                return 'Factura';
            case 'NC':
                return 'Nota crédito';
            case 'NCD':
                return 'Nota crédito por diferencia';
            case 'RV':
                return 'Remisión';
            default:
                return $type;
                break;
        }
    }

    /**
     * Metodo para obtener las sublineas del listado de referencias
     * 
     * @param string $condition Condicion 
     * 
     * @return array
     */
    public static function getSublineaReferencias($condition) {
        $arrayData = array();
        if (!empty($condition)) {

            try {
                $sql = "
                	SELECT 	LTRIM(RTRIM(T120.f120_referencia)) as referencia,T106.f106_id_plan , LTRIM(RTRIM(T106.f106_descripcion)) as sublinea
                        FROM t120_mc_items T120

                        LEFT JOIN t125_mc_items_criterios T125
                        ON T120.f120_rowid = T125.f125_rowid_item

                        LEFT JOIN t106_mc_criterios_item_mayores T106
                        ON T106.f106_id_plan = T125.f125_id_plan 
                        AND T106.f106_id = T125.f125_id_criterio_mayor

                        WHERE T120.f120_referencia in (" . $condition . ") and T106.f106_id_plan = 5

                        ORDER BY T120.f120_referencia							
				";

                $arrayData = Yii::$app->dbsql->createCommand($sql)->queryAll();
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }
        }
        return ArrayHelper::map($arrayData, 'referencia', 'sublinea');
    }

    /**
     * Metodo para obtener la informacion del vendedor para el nit y sucursal
     * seleccionados
     * 
     * @param integer $nit      Nit del cliente
     * @param string  $sucursal Id de la sucursal
     * 
     * @return type
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     */
    public static function getInfoVendedor($nit, $sucursal) {
        $data = [];
        if (!empty($nit) && is_numeric($nit) && !empty($sucursal)) {
            $sql = "
                SELECT TOP 1 dven.f200_razon_social as nombre, con.f015_email as email
                FROM t201_mm_clientes AS suc
                JOIN t200_mm_terceros AS cli ON cli.f200_rowid = suc.f201_rowid_tercero
                JOIN t210_mm_vendedores AS ven ON ven.f210_id = suc.f201_id_vendedor
                JOIN t200_mm_terceros AS dven ON dven.f200_rowid = ven.f210_rowid_tercero
                JOIN t015_mm_contactos AS con ON con.f015_rowid = dven.f200_rowid_contacto
                WHERE cli.f200_nit ='" . $nit . "' AND suc.f201_id_sucursal = '" . $sucursal . "'
                ";
            $data = Yii::$app->dbsql->createCommand($sql)->queryAll();
        }
        return $data;
    }
    
    /**
     * Metodo para obtener la informacion del vendedor para el nit y sucursal
     * seleccionados
     * 
     * @param integer $nit      Nit del cliente
     * @param string  $sucursal Id de la sucursal
     * 
     * @return type
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     */
    public static function getInventarioRef($referencia, $idBodega){
        $disponible = 0;        
        if(!empty($referencia) && !empty($idBodega)){
            
            try {
                $sql = "
                SELECT
                    T470.F470_ROWID_ITEM_EXT AS RItem,
                    T150.F150_ID As Bodega,
                    Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As disponible
                FROM
                    (T150_MC_BODEGAS T150 Right Outer Join
                       T470_CM_MOVTO_INVENT T470 
                    On T150.F150_ROWID = T470.F470_ROWID_BODEGA)

                JOIN t121_mc_items_extensiones ON f121_rowid = T470.F470_ROWID_ITEM_EXT
                JOIN t120_mc_items ON f120_rowid = f121_rowid_item

                WHERE
                    T470.F470_ID_fecha <= '".date('d-m-Y')."' 
                    AND T470.F470_IND_ESTADO_CM  <> '2' 
                    AND T470.F470_IND_TIPO_ITEM = 1
                    -- AND t470.f470_id_instalacion = '1'
                    AND T150.F150_ID = '".$idBodega."'
                    AND f120_referencia = '".$referencia."'
                        
                GROUP BY T470.F470_ROWID_ITEM_EXT, T150.F150_ID
                ";                                
                $data = Yii::$app->dbsql->createCommand($sql)->queryOne();
                if(count($data)>0){
                    $disponible = $data['disponible'];
                }
            } catch(Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
            }            
        }
        return $disponible;
    }
    
    /**
     * Retorna un array con la informacion de la bodega
     * 
     * @param string $cliente String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getCellar($cellar) {
        # Consulra a la base de datos SQL server -------------------------------
            $sql = "
            SELECT 
                f150_id as codigo, 
                (f150_id + ' - ' + f150_descripcion) as descripcion
            FROM t150_mc_bodegas
            WHERE f150_id IS NOT NULL AND f150_ind_estado = 1
            AND (f150_id LIKE '%$cellar%' OR f150_descripcion LIKE '%$cellar%') 
            GROUP BY f150_id, f150_descripcion
            ORDER BY f150_descripcion ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con la informacion de la bodega
     * 
     * @param string $cliente String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function listCellars() {
        # Consulra a la base de datos SQL server -------------------------------
            $sql = "
            SELECT 
                f150_id as codigo, 
                (f150_id + ' - ' + f150_descripcion) as descripcion
            FROM t150_mc_bodegas
            WHERE f150_id IS NOT NULL AND f150_ind_estado = 1
            GROUP BY f150_id, f150_descripcion
            ORDER BY f150_descripcion ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con la informacion de la bodega
     * 
     * @param string $cliente String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getInfoCellar($cellar) {
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "
            SELECT 
                f150_id as codigo, 
                (f150_id + ' - ' + f150_descripcion) as descripcion,
                f150_ind_multi_ubicacion,
                f150_ind_lotes,
                f150_ind_costos
            FROM t150_mc_bodegas
            WHERE f150_id IS NOT NULL AND f150_ind_estado = 1
            AND f150_id = '$cellar' 
            GROUP BY f150_id, f150_descripcion, f150_ind_multi_ubicacion,
                f150_ind_lotes,
                f150_ind_costos
            ORDER BY f150_descripcion ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryOne();
    }
    
    /**
     * Retorna un array con la informacion de la empresa
     * 
     * @param string $cia String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getCia($cia) {
        # Consulra a la base de datos SQL server -------------------------------
            $sql = "
            SELECT f010_id as codigo, (cast(f010_id as varchar) + ' - ' + f010_razon_social) as descripcion
            FROM t010_mm_companias
            WHERE f010_id IS NOT NULL AND f010_ind_estado = 1
            AND (f010_id LIKE '%$cia%' OR f010_razon_social LIKE '%$cia%') 
            GROUP BY f010_id, f010_razon_social
            ORDER BY f010_razon_social ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con los centros de operacion
     * 
     * @param string $cliente String de busqueda
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getListco($co) {
        # Consulra a la base de datos SQL server -------------------------------
            $sql = "
            SELECT f285_id as codigo, (f285_id + ' - ' + f285_descripcion) as descripcion
            FROM t285_co_centro_op
            WHERE f285_id IS NOT NULL AND f285_ind_estado = 1
            AND (f285_id LIKE '%$co%' OR f285_descripcion LIKE '%$co%') 
            GROUP BY f285_id, f285_descripcion
            ORDER BY f285_descripcion ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con la informacion (Nit,Razon social) del cliente
     * 
     * @param string $cia Id de la compañia
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getPdvs($cia) {
        $sql = "
            SELECT f200_nit as nit, (f200_nit + ' - ' + f200_razon_social) as razon_social
            FROM t200_mm_terceros
            JOIN t211_mm_funcionarios ON f211_rowid_tercero = f200_rowid
            WHERE f200_nit IS NOT NULL AND f200_nit != '' 
            -- AND f200_ind_estado = 1 AND f200_ind_interno = 1
            AND f200_id_cia = $cia 
            GROUP BY f200_nit, f200_razon_social
            ORDER BY f200_razon_social ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con la informacion de las unidades de negocio
     * 
     * @param string $cia Id de la compañia
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2022 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getBusinessUnits($cia) {
        $sql = "
            SELECT f281_id as id, (f281_id + ' - ' + f281_descripcion) as descripcion
            FROM t281_co_unidades_negocio
            WHERE f281_id_cia = $cia 
            GROUP BY f281_id, f281_descripcion
            ORDER BY f281_descripcion ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * Retorna un array con la informacion del funcionario 
     * 
     * @param string $nit Nit del PDV
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getCodeApplicant($nit) {
        $sql = "
            SELECT f211_id as codigo, f200_nit as nit, (f200_nit + ' - ' + f200_razon_social) as razon_social
            FROM t200_mm_terceros
            JOIN t211_mm_funcionarios ON f211_rowid_tercero = f200_rowid
            WHERE f200_nit IS NOT NULL AND f200_nit != '' 
            AND f200_ind_estado = 1 -- AND f200_ind_interno = 1
            AND f200_nit = '$nit'
            GROUP BY f211_id, f200_nit, f200_razon_social
            ORDER BY f200_razon_social ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryOne();
    }

}

?>
