<?php

namespace app\components;

use app\components\ERP;
use app\components\ClientSiesa;
use app\components\ClientTecno;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Clase para la integracion con el ERP siesa 
 *
 * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version Release: $Id$
 */
class ERPSiesa extends ERP {

    public $response;

    /**
     * Clase Parent
     * @var ParentClass 
     */
    private $_parentClass;

    /**
     * Constructor de la clase
     * @param ERP $parent
     */
    function __construct(ERP $parent = null) {
        if (isset($parent) && !is_null($parent)) {
            $this->_parentClass = $parent;
        }
    }

    /**
     * Metodo que retorna el listado de clientes de siesa 
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$     
     */
    public function getListClientes() {
        # Consultamos los clientes de siesa-------------------------------------
        $siesa = new ClientSiesa;
        $clientes = $siesa->getListClient();
        if (count($clientes) > 0) return $clientes;

        if ($this->_parentClass !== null) {
            return $this->_parentClass->getListClientes();
        }
    }

    /**
     * Retorna la informacion del cliente
     * 
     * @param integer $cliente Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$     
     */
    public function getCliente($cliente) {
        # Consultamos los clientes de siesa-------------------------------------
        $siesa = new ClientSiesa;
        $clientes = $siesa->getCliente($cliente);
        # Retornamos si encuentra el cliente  ----------------------------------
        if (count($clientes) > 0) return $clientes;

        if ($this->_parentClass !== null) {
            return $this->_parentClass->getListClientes();
        }
    }

    /**
     * Retorna las sucursales del cliente 
     * 
     * @param integer $nit Nit del cliente 
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$     
     */
    public function getSucursales($nit) {
        # Consultamos los clientes de siesa-------------------------------------
        $siesa = new ClientSiesa;
        $sucursales = $siesa->getSucursales($nit);
        # Retornamos si encuentra el cliente  ----------------------------------
        if (count($sucursales) > 0) return $sucursales;

        if ($this->_parentClass !== null) {
            return $this->_parentClass->getSucursales($nit);
        }
    }

    /**
     * Retorna los productos del cliente-sucursal
     * 
     * @param integer $nit
     * @param string $codSucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$     
     */
    public function getProductos($nit, $codSucursal) {
        # Consultamos los clientes de siesa-------------------------------------
        $siesa = new ClientSiesa;
        $productos = $siesa->getProductos($nit, $codSucursal);
        $nomSucursal = $siesa->getNameBranch($nit, $codSucursal);
        # Traemos el codigo de sucursal de Tecnocarnes por el nombre de 
        # la sucursal
        $tecno = new ClientTecno;
        $infoSucTecno = $tecno->getInfoBranch($nit, $nomSucursal);
        # Si trae datos consultamos en otros ERP--------------------------------
        if (isset($infoSucTecno['codigo_sucursal'])
                && !empty($infoSucTecno['codigo_sucursal'])
                && $this->_parentClass !== null) {
            $codSucursal = $infoSucTecno['codigo_sucursal'];
            $nProductos = $this->_parentClass->getProductos($nit, $codSucursal);
            $productos = array_merge($productos, $nProductos);
        }
        return $productos;
    }

    /**
     * Envia el pedido por web services al ERP SIESA
     * 
     * @param type $pedido
     * 
     * @return boolean
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$     
     */
    public function guardarPedido($pedido) {
        $envioSiesa = false;
        $envioTecno = false;
        # Guardamos el pedido en SIESA -----------------------------------------
        try {
            if (!empty($pedido) && is_object($pedido)) {
                $factoryOrder = new FactoryOrder();
                $arrayXml = $factoryOrder->createOrder($pedido->id, $pedido);                
                if (count($arrayXml) > 0) {
                    foreach ($arrayXml as $xml) {
                        if (!empty($xml)) {
                            $webServices = new WebServices;
                            $method = 'ImportarXML';
                            $url = \Yii::$app->params['paramsWs']['urlWs'];
                            $params = array('pvstrDatos' => $xml, 'printTipoError' => 0);
                            if ($webServices->executeMethodWs($method, $url,
                                            $params)) {
                                $response = $webServices->response;
                                if (isset($response->printTipoError)
                                        && $response->printTipoError === 0) {                                    
                                    $envioSiesa = true;
                                } else {
                                    $error = $response->ImportarXMLResult->any;                                    
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $xml . ']}', true) . '}',
                                            'webservices');
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $error . ']}', true) . '}',
                                            'webservices');
                                    $envioSiesa = false;
                                }
                            }
                        }
                    }
                }
            }            
        } catch (\Exception $exc) {
            $envioSiesa = false;
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r('{Error ['. $exc->getMessage() 
                    . ']}', true) . '}', 'webservices');
        }

        #Guardamos el pedido en los otros ERP ----------------------------------
        try {
            if ($this->_parentClass !== null) {            
                $envioTecno = $this->_parentClass->guardarPedido($pedido);
            }
        } catch (\Exception $exc) {
            echo $exc->getTraceAsString();
        }        
        # ----------------------------------------------------------------------        
        if(!$envioSiesa && !$envioTecno){
            return false;            
        }
        return true;
    }

}
