<?php

namespace app\components\ws_entry;

use Yii;
use yii\base\Component;
use app\models\Entry;
use app\models\Entrydetail;
use yii\helpers\ArrayHelper;
use app\components\ClientSiesa;
use app\components\ws_entry\HelperEntry;
use app\components\SummaryItems;

//mb_internal_encoding("utf-8");
/**
 * FactoryOrder.php
 *
 * PHP Version 5.3.0
 * 
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   SVN: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */

/**
 * FactoryOrder.php
 *
 * Clase para crear el xml del pedido
 *
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */
class FactoryEntry extends Component{

    /**
     * @const Constante para definir las variables de tipo int
     */
    const TYPE_INT = 'int';

    /**
     * @const Constante para definir las variables de tipo string
     */
    const TYPE_STRING = 'string';

    /**
     * Constructor de la clase
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S.     
     * @access public     
     */
    public function __construct() {
        
    }

    /**
     * Funcion desctructora 
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S. 
     * @access    public     
     */
    public function __destruct() {
        
    }

    /**
     * Metodo para crear la linea del servicio
     * 
     * @param ItemOrder $item Objeto de entrada
     * 
     * @return    string|boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _createLine($objectItem) {
        $itemOrder = new ItemEntry();
        $varsInt = $itemOrder->lehgthVarsInt();
        $varsString = $itemOrder->lehgthVarsString();
        $string = $this->_objectString($objectItem, $varsInt, $varsString);
        if (!empty($string) && $string) {
            $line = '<Linea>' . $string . '</Linea>';
            return $line;
        }
        return false;
    }

    /**
     * Metodo para crear la cabecera del pedido
     * 
     * @param object $objectHeader Objeto cabecera
     * 
     * @return    string|boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _createHeader($objectHeader) {
        $headerOrder = new HeaderEntry();
        $varsInt = $headerOrder->lehgthVarsInt();
        $varsString = $headerOrder->lehgthVarsString();
        $string = $this->_objectString($objectHeader, $varsInt, $varsString);
        if (!empty($string) && $string) {
            $line = '<Linea>' . $string . '</Linea>';
            return $line;
        }
        return false;
    }

    /**
     * Metodo para crear el pie de las lineas
     * 
     * @param int $countItem cantidad de items
     * 
     * @return string/boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _createFoot($countItem) {
        if (!empty($countItem) && is_int($countItem)) {
            $typeVar = self::TYPE_INT;
            $lenght = 7;
            $string = $this->_completeVar($countItem, $lenght, $typeVar);
            $complement = '<Linea>' . $string . '99990001001</Linea>';
            return $complement;
        }
        return false;
    }

    /**
     * Funcion para completar las variables 
     * 
     * @param string $value   Valor
     * @param int    $lenght  Tamaño
     * @param string $typeVar Tipo de variable
     * 
     * @return type
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _completeVar($value, $lenght, $typeVar) {
        $result = '';
        if (!empty($lenght) && is_int($lenght) && !empty($typeVar)) {
            if ($typeVar === self::TYPE_STRING) {
                //$value = utf8_decode($value);
                //$result = str_pad($value, $lenght, " ", STR_PAD_RIGHT);              
                $diff = strlen($value) - mb_strlen($value,
                                mb_detect_encoding($value));
                $result = str_pad($value, $lenght + $diff, " ", STR_PAD_RIGHT);
            } elseif ($typeVar === self::TYPE_INT) {
                $result = str_pad($value, $lenght, "0", STR_PAD_LEFT);
            }
        }
        return $result;
    }

    /**
     * Metodo para convertir el objeto 
     * en una cadena y completar con 0 o espacio segun 
     * el tipo de dato
     * 
     * @param object $objectItem Objeto
     * @param array  $varsInt    Array variables de tipo int
     * @param array  $varsString Array variables de tipo string
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _objectString($objectItem, $varsInt, $varsString) {
        $data = '';
        if (is_object($objectItem)) {
            foreach ($objectItem as $var => $value) {
                if (isset($varsInt[$var])) {
                    $typeVar = self::TYPE_INT;
                    $lenght = $varsInt[$var];
                } elseif (isset($varsString[$var])) {
                    $typeVar = self::TYPE_STRING;
                    $lenght = $varsString[$var];
                } else {
                    \Yii::error(__FILE__ . ':' . __LINE__ 
                            . 'El atributo {' . $var 
                            . '} no esta definido', 'webservices');
                    return false;
                }

                if ($var === 'cantPedidaBase') {
                    $value .= '.0000';
                }
                if ($var === 'precioUnitario') {
                    if(is_float($value + 0 )){
                        $tempValue = explode('.', $value);
                        $decimal = str_pad($tempValue[1], 4, "0", STR_PAD_RIGHT);
                        $value = $tempValue[0].'.'.$decimal;
                    }else{
                        $value .= '.0000';
                    }                    
                }
                $data .= $this->_completeVar($value, $lenght, $typeVar);
            }
        }
        return $data;
    }

    /**
     * Metodo que retorna el XML
     * basico para enviar los pedidos
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getBasicXML() {
        return '
            <Importar>  
                <NombreConexion>'
                . Yii::$app->params['paramsWs']['nameConection']
                . '</NombreConexion>  
                <IdCia>'
                . Yii::$app->params['paramsWs']['idCompania']
                . '</IdCia> 
                <Usuario>'
                . Yii::$app->params['paramsWs']['usuario']
                . '</Usuario> 
                <Clave>'
                . Yii::$app->params['paramsWs']['clave']
                . '</Clave> 
                <Datos>
                <Linea>000000100000001001</Linea>
                %__CABECERA__%
                %__LINEA__%
                %__FOOT__%                
                </Datos>
                </Importar>
            ';
    }

    /**
     * Metodo para crear el xml que sera 
     * enviado al servicio con la informacion del 
     * pedido
     * 
     * @param object $summaryItems Objeto con el array de items
     * 
     * @return    boolean/string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _createXML($objectHeader, $summaryItems) {
        if (is_object($summaryItems) && isset($summaryItems->arrayItems) 
                && count($summaryItems->arrayItems) > 0) {
            $arrayItems = $summaryItems->arrayItems;
            $summaryLine = '';
            $resultHeader = $this->_createHeader($objectHeader);

            if (empty($resultHeader) || !$resultHeader) {
                return false;
            }
            if (count($arrayItems) > 0) {
                $countLine = 3;
                foreach ($arrayItems as $key => $tempItem) {
                    $item = $tempItem;
                    $item->nroRegistro = count($arrayItems);
                    $item->numeroReg = $countLine;
                    $resultLine = $this->_createLine($item);
                    if (!empty($resultLine) && $resultLine) {
                        $countLine++;
                        $summaryLine .= $resultLine;
                    } else {
                        return false;
                    }
                }
            }

            $resultFoot = $this->_createFoot($countLine);
            $basicXml = $this->_getBasicXML();
            $arrayString = array('%__CABECERA__%', '%__LINEA__%', '%__FOOT__%');
            $arrayResult = array($resultHeader, $summaryLine, $resultFoot);
            $xml = str_replace($arrayString, $arrayResult, $basicXml);
            return $xml;
        }
        return false;
    }

    /**
     * Metodo para la creacion del xml 
     * de conexion 
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createXmlConnection() {
        return '
            <?xml version="1.0"?>	
            <Conexion>  
                <NombreConexion>'
                . Yii::$app->params['paramsWs']['nameConection']
                . '</NombreConexion> 
                <TipoBD>SQL</TipoBD> 
                <NombreBD>'
                . Yii::$app->params['paramsWs']['nombreDB']
                . '</NombreBD>
                <NombreServidorBD>'
                . Yii::$app->params['paramsWs']['nombreServidorDB']
                . '</NombreServidorBD>
                <UsuarioBD>'
                . Yii::$app->params['paramsWs']['usuarioDB']
                . '</UsuarioBD> 
                <ClaveBD>'
                . Yii::$app->params['paramsWs']['claveDB']
                . '</ClaveBD> 
                <NumConexionImpo>1</NumConexionImpo>
            </Conexion>
            ';
    }

    /**
     * Metodo que crea el XML del pedido a partir del modelo
     * Se $model no se envia o se envia NULL, la funcion 
     * calcula desde base de datos si esta existe, la carga 
     * y genera el xml
     * 
     * @param int   $idEntry Identificador de la orden
     * @param Order $model   Modelo tipo Order
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createEntry($idEntry, $model = null) {
        $arrayXml = [];
        try {
            if (!empty($idEntry) && is_numeric($idEntry)) {                             
                if (is_null($model)) {                    
                    $model = Entry::findOne($idEntry);
                }
                if (!empty($model)) {
                    $helperOrder = new HelperEntry();
                    $objectHeader = $helperOrder->createHeader($model);

                    $arrayOrderProduct = Entrydetail::find()
                            ->where(['entry_id'=>$idEntry])
                            ->all();   
                    $summaryItems = new SummaryItems();
                    if (count($arrayOrderProduct) > 0) {
                        $countItems = count($arrayOrderProduct);
                        //Consultamos todas las sublineas de las referencias pedidas
                        $references = [];
                        foreach ($arrayOrderProduct as $orderProduct) {
                            $references[] = $orderProduct->reference;
                        }
                        $clientSiesa = new ClientSiesa;
                        $cia = $model->id_cia;
                        $unitsMesure = $clientSiesa->getMesureUnit($references, $cia);
                        $arrayUnitMes = ArrayHelper::map($unitsMesure, 'f120_referencia', 'f120_id_unidad_orden');
                        
                        $idCellar = $model->id_cellar_in;
                        //traemos la info de la bodega si maneja lotes y ubicacion
                        $infoCellar  = $clientSiesa->getInfoCellar($idCellar);
                        $location = '';
                        $arrayLots = [];
                        $arrayCost = [];
                        
                        //Validamos las condiciones de la bodega
                        if($infoCellar){
                            //Validamos si usa ubicaciones la bodega de entrada
                            if(isset($infoCellar['f150_ind_multi_ubicacion']) 
                            && $infoCellar['f150_ind_multi_ubicacion'] === '1'){
                                $location = 'GENERICA';
                            }
                            //Validamos si maneja lotes la bodega de entrada 
                            if(isset($infoCellar['f150_ind_lotes']) 
                            && $infoCellar['f150_ind_lotes'] === '1'){
                                $lots = $clientSiesa->getLastLot($references, $cia, $model->id_cellar_in);
                                $arrayLots = ArrayHelper::map($lots, 'f120_referencia', 'lote');
                            }
                            //Validamos si maneja costos la bodega de entrada 
                            if(isset($infoCellar['f150_ind_costos']) 
                            && $infoCellar['f150_ind_costos'] === '1'){
                                $cost = $clientSiesa->getLastCost($references, $cia);
                                $arrayCost = ArrayHelper::map($cost, 'f120_referencia', 'costo');
                            }
                        }
                        
                        //Recopilamos informacion y creamos los item 
                        foreach ($arrayOrderProduct as $orderProduct) {
                            //Consultamos la bodega de cada item --------------- 
                            if(isset($arrayUnitMes[$orderProduct->reference])){
                                $unitMesure = $arrayUnitMes[$orderProduct->reference];
                            }else{
                                $unitMesure = 'UN';
                            }
                            
                            if(isset($arrayLots[$orderProduct->reference])){
                                $lot = $arrayLots[$orderProduct->reference];
                            }else{
                                $lot = '';
                            }
                            
                            if(isset($arrayCost[$orderProduct->reference])){
                                $cost = $arrayCost[$orderProduct->reference];
                            }else{
                                $cost = 0;
                            }
                            
                            $summaryItems->arrayItems[] = $helperOrder->createItem(
                                $orderProduct, $countItems, $idCellar, 
                                    $unitMesure, $lot, $cost, $location);
                        }
                    }                    
                    //Creamos XML para los demas -------------------------------
                    $xml = $this->_createXML($objectHeader, $summaryItems);
                    $arrayXml[] = str_replace("&amp;", "&", $xml);                                        
                }
            }
        } catch (Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                . print_r($exc->getMessage(), true) . '}', 'webservices'); 
        }        
        return $arrayXml;
    }

}

?>
