<?php

namespace app\components\ws_entry;

use Yii;
use yii\base\Component;
use app\components\ClientSiesa;
use app\components\ws_entry\HeaderEntry;
use app\components\ws_entry\ItemEntry;

/**
 * HelperOrder.php
 *
 * PHP Version 5.3.0
 * 
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   SVN: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */

/**
 * HelperOrder.php
 *
 * // SUS COMENTARIOS AQUI
 *
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */
class HelperEntry extends Component {

    private $_count = 2;
    private $_idCo;
    private $_idMotivo;
    private $_fechaEntrega;
    private $_diasEntrega;
    private $_clientSiesa;
    private $_lot;

    //Constantes para la cabecera

    const NUMERO_REG_HEADER = '2';
    const TIPO_REG_HEADER = '450';
    const SUBTIPO_REG = '00';
    const VERSION_REG = '02';
    const CIA = '1';
    const CONSEC = '1';
    const CONSEC_AUTO_REG = '1';
    const ID_TIPO_DOC = 'CEI'; //CEI: ENTRADA INVENTARIO, CEA: ENTRADA ALMACEN
    const ID_CLASE_DOCTO = '61';
    const IND_ESTADO = '1';
    const ID_CONCEPTO_HEADER = '601';
        
    const NUM_DOCTO_REFERENCIA = '';
    const REFERENCIA = '';
    const ID_CARGUE = '';
    const IND_IMPRESO = '0';
    //Constantes para los items
    const VERSION_REG_ITEM = '05';
    const TIPO_REG_ITEM = '470';
    const ID_CONCEPTO = '601';
    const ID_ITEM = '';
    const ID_EXT1_DETALLE = '';
    const ID_EXT2_DETALLE = '';
    const CODIGO_BARRAS = '';
    const ID_UN_MOVTO = '001';
    const ID_CCOSTO_MOVTO = '';
    const ID_PROYECTO = '';
    const PRECIO_UNITARIO = '0';
    const CANT_2_PEDIDA = '000000000000000.0000';

    /**
     * Constructor de la clase
     * 
     * @return    void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S.     
     * @access    public     
     */
    public function __construct() {
        
    }

    /**
     * Funcion desctructora 
     * 
     * @return    void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2017 INGENEO S.A.S. 
     * @access    public     
     */
    public function __destruct() {
        
    }

    /**
     * Metodo para crear la cabecera del pedido
     * 
     * @param Order $model Modelo Order
     * 
     * @return boolean|\HeaderOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createHeader($model) {
        if (is_object($model)) {
            try {
                $nit = $model->nit;
                $this->_idMotivo = '01';
                $this->_idCo = $model->id_co;
                $this->_clientSiesa = new ClientSiesa();
                //$this->_lot = $this->_clientSiesa->getLastLot($model->id_cia);

                $solicitante = $model->getSolicitante($model->nit);
                $codigo = isset($solicitante['codigo'])? $solicitante['codigo'] : $nit;
                //--------------------------------------------------------------
                $headerOrder = new HeaderEntry();
                $headerOrder->numeroReg = self::NUMERO_REG_HEADER;
                $headerOrder->tipoReg = self::TIPO_REG_HEADER;
                $headerOrder->subTipoReg = self::SUBTIPO_REG;
                $headerOrder->versionReg = self::VERSION_REG;
                $headerOrder->cia = $model->id_cia;
                $headerOrder->consecAutoReg = self::CONSEC_AUTO_REG;
                $headerOrder->idCo = $this->_idCo; 
                $headerOrder->idTipoDocto = self::ID_TIPO_DOC; 
                $headerOrder->consecDocto = self::CONSEC;
                $headerOrder->idFecha = date('Ymd'); //Formatro AAAAMMDD
                $headerOrder->idTercero = $solicitante['nit'];                
                //$headerOrder->idTercero = $nit;                
                $headerOrder->idClaseDoct = self::ID_CLASE_DOCTO;
                $headerOrder->indEstado = self::IND_ESTADO;
                $headerOrder->indImpresion = self::IND_IMPRESO;
                $headerOrder->notas = 'No. Entrada APP Chefbrger: '
                        . $model->id . ' -- ' . $model->description;
                $headerOrder->idConcepto = self::ID_CONCEPTO_HEADER;
                $headerOrder->idBodegaSalida = $model->id_cellar_out;
                $headerOrder->idBodegaEntrada = $model->id_cellar_in;
                $headerOrder->doctoAlterno = 'REQ: '.$model->number;
                $headerOrder->idCoBase = '';
                $headerOrder->idTipoDoctoBase = '';
                $headerOrder->consecDoctoBase = '';
                $headerOrder->idVehiculo = '';
                $headerOrder->idTerceroTransp = '';
                $headerOrder->idSucursalTransp = '';
                $headerOrder->idTerceroConductor = '';
                $headerOrder->nombreConductor = '';
                $headerOrder->identifConductor = '';
                $headerOrder->numeroGuia = '';
                $headerOrder->cajas = '';
                $headerOrder->peso = '';
                $headerOrder->volumen = '';
                $headerOrder->valorSeguros = '';
                $headerOrder->notasTransp = '';
                return $headerOrder;
            } catch (\Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                return false;
            }
        }
        return false;
    }

    /**
     * Metodo para crear el objeto de items
     * 
     * @param OrderProduct $model      Modelo de tipo OrderProduct
     * @param int          $countItems Contador de items
     * 
     * @return boolean|\ItemOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function createItem($model, $countItems, $idBodega, $unitMesure, 
            $lot, $cost, $location) {

        if (is_object($model)) {
            try {
                $error = false;

                if (!$idBodega) $error = true;
                if ($error) return false;

                $this->_count = $this->_count + 1;

                $item = new ItemEntry();
                $item->numeroReg = $this->_count; //Consecutivo del item
                $item->tipoReg = self::TIPO_REG_ITEM;
                $item->subtipoReg = self::SUBTIPO_REG;
                $item->versionReg = self::VERSION_REG_ITEM;
                $item->cia = self::CIA; // id de la compañia 
                $item->idCo = $this->_idCo; 
                $item->idTipoDocto = self::ID_TIPO_DOC; 
                $item->consecDocto = self::CONSEC; 
                $item->nroRegistro = $countItems;
                $item->f_campo = '';
                $item->idBodega = $idBodega; 
                $item->idUbicacionAux = $location;
                $item->idLote = $lot;
                $item->idConcepto = self::ID_CONCEPTO;
                $item->idMotivo = $this->_idMotivo;
                $item->idCoMovto = $this->_idCo;
                $item->campo = 'AP';
                $item->idCcostoMovto = self::ID_CCOSTO_MOVTO;
                $item->idProyecto = self::ID_PROYECTO;
                $item->idUnidadMedida = $unitMesure;
                $item->cantPedidaBase = $model->quantity;
                $item->cant2pedida = self::CANT_2_PEDIDA;
                $item->precioUnitario = $cost;
                $item->notas = '';
                $item->descVariable = '';
                $item->descItem = '';
                $item->idUmInventario = '';
                $item->idUbicaciónAuxEnt = $location;
                $item->idLoteEnt = $lot;
                $item->idItem = self::ID_ITEM;
                $item->referenciaItem = $model->reference; 
                $item->codigoBarras = self::CODIGO_BARRAS;
                $item->idExt1Detalle = self::ID_EXT1_DETALLE;
                $item->idExt2Detalle = self::ID_EXT2_DETALLE;
                $item->idUnMovto = self::ID_UN_MOVTO;
                return $item;
            } catch (Exception $exc) {
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r($exc->getMessage(), true) . '}', 'webservices');
                return false;
            }
        }
    }
    
    /**
     * Retorna el Id de centro de operaciones
     * 
     * @param Order $model Modelo Order
     * 
     * @return boolean|\HeaderOrder
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getIdCo(){
        return $this->_idCo;
    }

}

?>
