<?php

namespace app\controllers;

use Yii;
use app\models\Configpdv;
use app\models\ConfigpdvSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Json;

/**
 * ConfigpdvController implements the CRUD actions for Configpdv model.
 */
class ConfigpdvController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Configpdv models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ConfigpdvSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Configpdv model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Configpdv model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Configpdv();
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Configpdv model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Configpdv model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Configpdv model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Configpdv the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Configpdv::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetcellar($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Configpdv();            
            $cellars = $model->getCellar($q);                         
            if(count($cellars)>0){                   
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];  
                }                 
            }            
        }
        elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna los centros de operacion
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetlistco($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Configpdv();            
            $cellars = $model->getListco($q);                         
            if(count($cellars)>0){                   
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];  
                }                 
            }            
        }
        elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetcia($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Configpdv();            
            $cellars = $model->getCia($q);                         
            if(count($cellars)>0){                   
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];  
                }                 
            }            
        }elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna las sucursales      
     * 
     * @return type
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetpdvs() {
        $out = [];
        if (isset($_POST['depdrop_all_params']['configpdv-id_cia'])) {
            $cia = $_POST['depdrop_all_params']['configpdv-id_cia'];            
            if ($cia != null) {
                $model = new Configpdv();            
                $pdvs = $model->getPdvs($cia);                 
                if(count($pdvs)>0){                   
                    foreach ($pdvs as $pdv) {
                        $out[] = ['id' => $pdv['nit'], 
                            'name' => $pdv['razon_social']];  
                    }                 
                } 
                return Json::encode(['output'=>$out, 'selected'=>'']);
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
    }
    
    /**
     * Retorna las sucursales      
     * 
     * @return type
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetBusinessUnit() {
        $out = [];
        if (isset($_POST['depdrop_all_params']['configpdv-id_cia'])) {
            $cia = $_POST['depdrop_all_params']['configpdv-id_cia'];            
            if ($cia != null) {
                $model = new Configpdv();            
                $businessUnits = $model->getBusinessUnits($cia);                
                if(count($businessUnits)>0){                   
                    foreach ($businessUnits as $businessUnit) {
                        $out[] = ['id' => $businessUnit['descripcion'], 
                            'name' => $businessUnit['descripcion']];  
                    }
                } 
                echo Json::encode(['output'=>$out, 'selected'=>'']);
                return;
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
    }
    
}
