<?php

namespace app\controllers;

use Yii;
use app\models\Entry;
use app\models\EntrySearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\Entrydetail;
use yii\helpers\Json;
use yii\helpers\HtmlPurifier;


/**
 * EntryController implements the CRUD actions for Entry model.
 */
class EntryController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Entry models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new EntrySearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Entry model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Entry model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Entry();

        if ($model->load(Yii::$app->request->post())) {                    
            $this->_postSave($model);
            return $this->render('create', [
                'model' => $model,                
            ]);
        } else {
            return $this->render('create', [
                'model' => $model,                
            ]);
        }
    }

    /**
     * Updates an existing Entry model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post())) {            
            $this->_postSave($model);
            return $this->render('update', [
                'model' => $model,
            ]);
        } else {
            if($model->status == Entry::STATUS_ALLOW){
                return $this->redirect(['index']);
            }else{
                return $this->render('update', [
                    'model' => $model,
                ]);
            }            
        }
    }

    /**
     * Deletes an existing Entry model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Entry model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Entry the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Entry::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
    
    /**
     * Retorna el cliente buscado
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetclient($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Entry();            
            $clients = $model->getClient($q);                         
            if(count($clients)>0){                   
                foreach ($clients as $client) {
                    $data[] = ['id' => $client['nit'], 'text' => $client['razon_social']];  
                }                 
            }            
        }
        elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna las sucursales      
     * 
     * @return type
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetbranches() {
        $out = [];
        if (isset($_POST['depdrop_all_params']['requisition-nit'])) {
            $nit = $_POST['depdrop_all_params']['requisition-nit'];            
            if ($nit != null) {
                $model = new Entry();            
                $sucursales = $model->getBranches($nit);                 
                if(count($sucursales)>0){                   
                    foreach ($sucursales as $sucursal) {
                        $out[] = ['id' => $sucursal['cod_sucursal'], 
                            'name' => $sucursal['nombre_sucursal']];  
                    }                 
                }                
                echo Json::encode(['output'=>$out, 'selected'=>'']);
                return;
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
    }
    
    /**
     * Retorna el listado de productos 
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetproducts() {
        if (isset($_POST['nit']) && !empty($_POST['nit'])) {            
            $ancho = HtmlPurifier::process($_POST['ancho']);
            $pedido = HtmlPurifier::process($_POST['pedido']);
            $cia = HtmlPurifier::process($_POST['cia']);
            $nit = HtmlPurifier::process($_POST['nit']);
            $model = new Entry(); 
            $listProduct = $model->getProducts($nit, $cia);
            return $this->renderAjax('_formProducts', [
                        'model' => $model,
                        'listProduct' => $listProduct,
                        'ancho' => $ancho,
                        'pedido' => $pedido,
            ]);          
        }        
    }
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetcellar($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Entry();            
            $cellars = $model->getCellar($q);                         
            if(count($cellars)>0){                   
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];  
                }                 
            }            
        }
        elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetcellarin($nit) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        $pdv = \app\models\Configpdv::find()->where(['code_pdv'=>$nit])
                ->one();
        if(!empty($pdv)){
            $data = ['id_cellar_in'=>$pdv->id_cellar_in, 'name_cellar_in'=>$pdv->name_cellar_in];
        }
        return $data;
    }
    
    /**
     * Retnorna los botones
     * 
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionChangebuttom() {
        if (isset($_POST['type']) && $_POST['type'] === 'liquidar') {
            return $this->renderAjax('_buttomLiquidar');        
        } else {
            return $this->renderAjax('_buttomSend');
        }
    }
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetcia($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            # Buscamos el cliente en todos los                                 
            $model = new Entry();            
            $cellars = $model->getCia($q);                         
            if(count($cellars)>0){                   
                foreach ($cellars as $cellar) {
                    $data[] = ['id' => $cellar['codigo'], 'text' => $cellar['descripcion']];  
                }                 
            }            
        }elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna las bodegas
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetinfopdv() {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        if(isset($_POST['nit'])  && !empty($_POST['nit'])
            && isset($_POST['business_name'])  && !empty($_POST['business_name'])){
            $nit = HtmlPurifier::process($_POST['nit']);
            $tempBusinessName = HtmlPurifier::process($_POST['business_name']);
            $business_name = explode(' - ', $tempBusinessName);
            $requisition = new Entry;
            return $requisition->getInfopdv($nit, $business_name[1]);
        }else{
            return false;
        }
    }
    
    /**
     * Metodo para guardar las compras 
     * 
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    private function _postSave(&$model){
        //Captura de datos Post ------------------------------------------------        
        $msg = '';
        if (isset($_POST['Entry'])) {            
            //Validacion de status segun botones -------------------------------
            if (isset($_POST['status']) && $_POST['status'] === Entry::STATUS_PENDING) {
                $model->status = Entry::STATUS_PENDING;
            } elseif (isset($_POST['status']) && $_POST['status'] === Entry::STATUS_ALLOW) {
                $model->status = Entry::STATUS_ALLOW;
            } else {
                $model->status = Entry::STATUS_PARCIAL;
            }
            //------------------------------------------------------------------
            $razonSocial = explode(' - ',$model->business_name);
            $model->nit = isset($razonSocial[0]) ? $razonSocial[0] : null;            
            $model->business_name = isset($razonSocial[1]) ? $razonSocial[1] : null;
            //------------------------------------------------------------------
            if ($model->validate()) {
                $quantitys = $_POST['cantidad']; 
                if (count($quantitys) > 0) {
                    // Guardar los datos del pedido ----------------------------                                        
                    $model->save();
                    //----------------------------------------------------------
                    $error = true; 
                    //Variable para indicar al vendedor en caso de no haber ----
                    $entryDetail = Entrydetail::deleteAll(['entry_id'=>$model->id]);
                    foreach ($quantitys as $key => $value) {
                        if (!is_numeric($value)) {
                            $value = 0;
                        }     
                        
                        if (!empty($value) && $value > 0) {                            
                            $error = false;
                            $undExist =  '';                            
                            $tempReferencia = explode('-',$key);
                            $referencia = $tempReferencia[1];                            
                            $description = HtmlPurifier::process($_POST[$key . '_description']);
                            $idCategory = HtmlPurifier::process($_POST[$key . '_cat']);                                         
                            //--------------------------------------------------
                            if(isset($_POST[$key . '_inv'])){
                                $undExist = HtmlPurifier::process($_POST[$key . '_inv']);
                            }                                                                                                                                                                      
                            //Guardamos las referencias del pedido -------------                             
                            $entryDetail = new Entrydetail;                            
                            $entryDetail->entry_id = $model->id;
                            $entryDetail->reference = $referencia;
                            $entryDetail->description = $description;
                            $entryDetail->quantity = $value;
                            $entryDetail->category = $idCategory;                            
                            if(!$entryDetail->save()){ 
                                Yii::$app->session->setFlash('danger', 'Error al guardar los productos');
                            }
                        }                        
                    }                     
                    $model->save();                    
                    //----------------------------------------------------------
                    if ($error) {                        
                        Yii::$app->session->setFlash('danger', 
                                'No hay cantidades ingresadas');
                        $model->delete();
                    } else {
                        //Envio de correo a los pedidos aprobados --------------
                        if ($model->status === Entry::STATUS_ALLOW) {
                            if(!$model->sendEntryERP($model)){                                                        
                                $model->status = Entry::STATUS_PENDING;
                                $model->save();                                              
                                $msg = [
                                    'tipo'=>'success', 
                                    'msg'=>Yii::t('app', 'Error guardado entrada')
                                    . ' - No. PEDIDO: '.$model->id
                                ];
                            }else{
                                Yii::$app->session->setFlash('success', 
                                    'Entrada enviada con exito'. ' - No. PEDIDO: '
                                        .$model->id);
                                $msg = [
                                    'tipo'=>'success', 
                                    'msg'=>Yii::t('app', 'Exito al guardar entrada')
                                    . ' - No. PEDIDO: '.$model->id
                                ];
                            }    
                            $this->redirect(['entry/view', 'id' => $model->id, 'msg'=>$msg]); 
                            \Yii::$app->end();
                        }
                        //Notificacion al solicitar aprobacion -----------------
                        if ($model->status === Entry::STATUS_PENDING) {                            
                            Yii::$app->session->setFlash('success', 
                                    'Entrada guardada como '.Entry::STATUS_PENDING);
                            $msg = [
                                    'tipo'=>'success', 
                                    'msg'=>Yii::t('app', 'Entrada guardada como '.Entry::STATUS_PENDING)
                                    . ' - No. ENTRADA: '.$model->id
                                ];
                            $this->redirect(['entry/view', 'id' => $model->id, 'msg'=>$msg]);
                            \Yii::$app->end();
                        }                        
                    }
                }
            }
        }
    }
}
