<?php

namespace app\models;

use Yii;
use app\components\ClientSiesa;
use yii\db\ActiveRecord;
use yii\behaviors\BlameableBehavior;
use yii\db\Expression;
use app\components\FactoryOrder;
use app\components\WebServices;

/**
 * This is the model class for table "requisition".
 *
 * @property int $id
 * @property string $nit
 * @property string $business_name
 * @property string $code_branch
 * @property string $branch_name
 * @property string $num_req
 * @property string $description
 * @property string $status
 * @property string $date
 * @property string $total_neto
 * @property string $total_iva
 * @property string $total
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 *
 * @property RequisitionDetail[] $requisitionDetails
 */
class Requisition extends ActiveRecord
{
    const STATUS_PENDING = 'pendiente';
    const STATUS_ALLOW = 'autorizado';
    const STATUS_PARCIAL = 'parcial';
    const STATUS_DELETE = 'borrado';
    const ORDER_NEW = 'nuevo';
    const USR_ADMIN = 'admin';
    
    public $id_pdv;
    public $cellar_in;
    public $cia;
	public $date_range;
	public $percentage;
	public $datetime_start;
	public $datetime_end;
    
    /**
     * Aplica los valores previos del modelo al activerecord
     * @return array
     */
    public function behaviors() {
        return [
            'timestamp' => [
                'class' => 'yii\behaviors\TimestampBehavior',
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created', 'modified'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified'],
                ],
                'value' => new Expression('NOW()'),
            ],
            'blameable' => [
                'class' => BlameableBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_INSERT => ['created_by', 'modified_by'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['modified_by'],
                ],
                'value' => isset(Yii::$app->user->identity->username) ? Yii::$app->user->identity->username
                            : '',
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'requisition';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['nit', 'business_name', 'status', 'date', 'id_cellar_out', 
                'id_cellar_in', 'cellar_in', 'id_co', 'name_co', 'id_cia', 
                'name_cia', 'date_range', 'business_unit'], 'required'],
            [['description', 'name_co', 'name_cia', 'date_range', 'datetime_start', 'datetime_end'], 'string'],
            [['date', 'created', 'modified', 'percentage', 'datetime_start', 'datetime_end'], 'safe'],
            [['nit', 'status', 'total_neto', 'total_iva', 'total', 
                'id_cellar_out', 'id_cellar_in', 'business_unit'], 'string', 'max' => 45],
            [['business_name', 'code_branch', 'branch_name', 'num_req', 
                'name_cellar_out', 'name_cellar_in'], 'string', 'max' => 250],
            [['created_by', 'modified_by'], 'string', 'max' => 150],
            [['percentage'], 'match' ,'pattern'=>'/^\d+([\.,]\d+)?$/', 'message'=> 'Solo se permiten ingresar números.'], 
        ];
    }
    
    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'nit' => Yii::t('app', 'Nit'),
            'cia' => Yii::t('app', 'Cia'),
            'business_name' => Yii::t('app', 'Business Name'),
            'code_branch' => Yii::t('app', 'Code Branch'),
            'branch_name' => Yii::t('app', 'Branch Name'),
            'id_cellar_out' => Yii::t('app', 'Id Cellar Out'),
            'name_cellar_out' => Yii::t('app', 'Name Cellar Out'),
            'cellar_in' => Yii::t('app', 'Cellar In'),
            'id_cellar_in' => Yii::t('app', 'Id Cellar In'),
            'name_cellar_in' => Yii::t('app', 'Name Cellar In'),
            'num_req' => Yii::t('app', 'Num Req'),
            'description' => Yii::t('app', 'Description'),
            'status' => Yii::t('app', 'Status'),
            'date' => Yii::t('app', 'Date'),
            'business_unit' => Yii::t('app', 'Business Unit'),
            'total_neto' => Yii::t('app', 'Total Neto'),
            'total_iva' => Yii::t('app', 'Total Iva'),
            'total' => Yii::t('app', 'Total'),
            'created' => Yii::t('app', 'Created'),
            'created_by' => Yii::t('app', 'Created By'),
            'modified' => Yii::t('app', 'Modified'),
            'modified_by' => Yii::t('app', 'Modified By'),
            'date_range' => Yii::t('app', 'Fecha pedidos'),
            'percentage' => Yii::t('app', '% incremento'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getRequisitionDetails()
    {
        return $this->hasMany(RequisitionDetail::className(), ['requisition_id' => 'id']);
    }
    
    /**
     * Retorna el listado de clientes
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getLisClients(){
        $integradorERP = new IntegradorERP();
        return $integradorERP->getListClientes();
    }
    
    /**
     * Retorna la informacion del cliente
     * 
     * @param string $cliente Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getClient($cliente = null){
        //$siesa = new ClientSiesa;
        //return $siesa->getCliente($cliente);
        $data = [];
        $clients = Configpdv::find()->joinWith('usrPdvs')
                ->where(['user_id'=>Yii::$app->user->id])->asArray()->all();
        if(count($clients)>0){                   
            foreach ($clients as $client) {
                $data[$client['code_pdv']] = $client['code_pdv'].' - '.$client['name_pdv'];    
            }                 
        } 
        return $data;
    }
    
    /**
     * Obtiene las sucursales a partir del nit
     * 
     * @param string $nit Nit del cliente
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getBranches($nit){
        $integradorERP = new ClientSiesa();        
        return $integradorERP->getSucursales($nit);
    }
    
    /**
     * Retorna los productos asociados a un nit y sucursal
     * 
     * @param string $nit         Nit de la sucursal
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getProducts($nit, $cia, $date_start, 
			$date_end, $percentage, $cellar_in){
        $siesa = new ClientSiesa;
        return $siesa->getProductos($nit, $cia, $date_start, 
			$date_end, $percentage, $cellar_in);
    }
    
    /**
     * Envia el pedido a los diferente ERP
     * 
     * @param Pedidos $model
     * 
     * @return boolean
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function guardarPedidoERP($model){
		$envioSiesa = false;
        try {
            if (!empty($model) && is_object($model)) {
                $factoryOrder = new FactoryOrder();
                $arrayXml = $factoryOrder->createOrder($model->id, $model);
	
				\Yii::error(__FILE__ . ':' . __LINE__ . 'XML SIESA {'. print_r($arrayXml, true) . '}');
                // echo '<pre>' . __FILE__ . ':' . __LINE__ . '{' . print_r($arrayXml , true) . '}';
                // exit;

                if (count($arrayXml) > 0) {
                    foreach ($arrayXml as $xml) {
                        if (!empty($xml)) {
                            $webServices = new WebServices;
                            $method = 'ImportarXML';
                            $url = \Yii::$app->params['paramsWs']['urlWs'];
                            $params = array('pvstrDatos' => $xml, 'printTipoError' => 0);
                            if ($webServices->executeMethodWs($method, $url,
                                            $params)) {
                                $response = $webServices->response;
                                if (isset($response->printTipoError)
                                        && $response->printTipoError === 0) {                                    
                                    $envioSiesa = true;
                                } else {
                                    $error = $response->ImportarXMLResult->any;                                    
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $xml . ']}', true) . '}',
                                            'webservices');
                                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                            . print_r('{Error xml ['
                                                    . $error . ']}', true) . '}',
                                            'webservices');
                                    $envioSiesa = false;
                                }
                            }
                        }
                    }
                }
            }            
        } catch (\Exception $exc) {
            $envioSiesa = false;
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r('{Error ['. $exc->getMessage() 
                    . ']}', true) . '}', 'webservices');
        }  
        return $envioSiesa;
    }
    
    /**
     * Metodo para calcular el numero de dias 
     * entre la fecha de entrega y la fecha actual del sistema
     * 
     * @param string $fecha Fecha de entrega
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   SVN: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getNumDiasEntrega($fecha)
    {
        if(!empty($fecha) && is_string($fecha)) {
            $segundos = strtotime($fecha) - strtotime(date('Y-m-d'));
            return abs(intval($segundos / 60 / 60 / 24));
        }
        return false;
    }
    
    /**
     * Retorna el 
     * 
     * @param string $fecha Fecha de entrega
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   SVN: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getSolicitante($nit)
    {
        $siesa = new ClientSiesa;
        return $siesa->getCodeApplicant($nit);
    }
    
    /**
     * Retorna la informacion de las bodegas
     * 
     * @param string $cellar Bodega
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getCellar($cellar){
        $siesa = new ClientSiesa;
        return $siesa->getCellar($cellar);
    }
    
    /**
     * Retorna la informacion de las bodegas
     * 
     * @param string $cellar Bodega
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function listCellars(){        
        $data = [];
        $siesa = new ClientSiesa;
        $cellars = $siesa->listCellars();
        if(count($cellars)>0){                   
            foreach ($cellars as $cellar) {
                $data[$cellar['codigo']] = $cellar['descripcion'];    
            }                 
        } 
        return $data;
    }
    
    
    
    /**
     * Retorna la informacion del punto de venta configurado
     * 
     * @param string $pdv            Nit del punto de venta
     * @param string $$business_name Nombre del punto de venta
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getInfopdv($pdv, $business_name){
        return Configpdv::find()->joinWith('usrPdvs')
                ->where(['user_id'=>Yii::$app->user->id, 'code_pdv'=>$pdv, 
                    'name_pdv'=>$business_name])
                ->asArray()->one(); 
    }
    
    
}
