<?php

namespace app\models;

use Yii;
use app\models\User;
use app\models\Configpdv;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "usr_pdv".
 *
 * @property int $id
 * @property int $user_id
 * @property int $config_pdv_id
 *
 * @property ConfigPdv $configPdv
 * @property User $user
 */
class Usrpdv extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'usr_pdv';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['user_id', 'config_pdv_id'], 'required'],
            [['user_id', 'config_pdv_id'], 'integer'],
            [['config_pdv_id'], 'exist', 'skipOnError' => true, 'targetClass' => ConfigPdv::className(), 'targetAttribute' => ['config_pdv_id' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['user_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'user_id' => Yii::t('app', 'User ID'),
            'config_pdv_id' => Yii::t('app', 'Config Pdv ID'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getConfigpdv()
    {
        return $this->hasOne(ConfigPdv::className(), ['id' => 'config_pdv_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(User::className(), ['id' => 'user_id']);
    }
    
    /**
     * Retorna el listado de usuarios
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getUsersDrop(){
        return ArrayHelper::map(User::find()->where(['status'=>10])->all(), 'id', 'username');
    }
    
    /**
     * Consulta los puntos de venta 
     * 
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function getPdvsDrop(){
        $resul = [];
        $pdvs = Configpdv::find()->all();
        if(count($pdvs)>0){
            foreach ($pdvs as $pdv) {
                $resul[$pdv->id] = $pdv->code_pdv.' - '.$pdv->name_pdv;
            }
        }
        return $resul;
    }
}
